/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"

#define	EXTERN	extern
#include "pam.h"
#include "bus.h"
#include "processor.h"
#include "Virtual_h8.h"
#include "globals.h"
#include "Display.h"




#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// Display

Display::Display()
{
	m_bmpBackground.LoadBitmap(IDB_MAN1_LABELS);
	m_bInterruptEnabledLED = FALSE;
	m_bPowerLED = FALSE;
	m_bRunLED = FALSE;
}

Display::~Display()
{
}


BEGIN_MESSAGE_MAP(Display, CWnd)
	//{{AFX_MSG_MAP(Display)
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Display message handlers


/*

	This code assumes that the parent contains the following definitions:

#define	DISPLAY_WIDTH	600
#define	DISPLAY_HEIGHT	115

	The actual definitions are:

#define	DISPLAY_WIDTH	525
#define	DISPLAY_HEIGHT	91


The LED displays are 5/8 inch high.  The window is 7-7/8 x 1-3/8

This makes the LEDs an array of 26x42 pixels.


	We need to compute the dimensions of the display elements from the
	actual size of the window and bit-blt the bitmap IDB_MAN1_LABELS to suit.
	Or, we need to build all this in a memory bitmap context and StretchBlt
	to fit the actual dimensions.

*/



void Display::OnPaint() 
{
	CPaintDC		dc(this); // device context for painting
	CBrush *		pRedBrush = new CBrush(RGB(255, 0, 0));
	CPen *			pRedPen = new CPen(PS_SOLID, 5, RGB(255, 0, 0));
	CBrush *		pBackground = new CBrush(RGB(102,0,0));
	CRect			rWork;
	CRect			rSegment;
	int				nDCIdent;
	int				nDigit;
	const int		nDigitXOrigin[9] =	{115, 153, 190,
										 245, 283, 320,
										 400, 438, 475
										};
	const int		nDigitYOrigin = 25;
	const CRect		rPowerLED(30, 73, 43, 85);
	const CRect		rRunLED(30, 53, 43, 65);
	const CRect		rMonitorLED(30, 33, 43, 45);
	const CRect		rInterruptEnableLED(30, 13, 43, 25);
	const CRect		rSegA(7, 0, 25, 2);
	const CRect		rSegB(20, 1, 25, 20);
	const CRect		rSegC(17, 20, 22, 40);
	const CRect		rSegD(0, 39, 18, 41);
	const CRect		rSegE(0, 20, 5, 40);
	const CRect		rSegF(3, 1, 8, 20);
	const CRect		rSegG(3, 19, 22, 21);
	const CRect		rSegP(23, 39, 25, 41);
	CDC 			memDC;
	CBitmap *		pBmpOld;
	CRect			rWnd;

	nDCIdent = dc.SaveDC();
	GetClientRect(&rWnd);
	memDC.CreateCompatibleDC(&dc);
	pBmpOld = memDC.SelectObject(&m_bmpBackground);

//	Show the current states of the Power LED, RUN LED, MON LED and IEN LED as
//	either RED or BACKGROUND, based on their BOOL member variables' states.

	memDC.SelectStockObject(NULL_PEN);
	if (m_bPowerLED)
	{
		memDC.SelectObject(pRedBrush);
		memDC.Ellipse(rPowerLED);
		DeleteObject(pRedBrush);
	}
	else
	{
		memDC.SelectObject(pBackground);
		memDC.Ellipse(rPowerLED);
		DeleteObject(pBackground);
	}
	if (m_bRunLED)
	{
		memDC.SelectObject(pRedBrush);
		memDC.Ellipse(rRunLED);
		DeleteObject(pRedBrush);
	}
	else
	{
		memDC.SelectObject(pBackground);
		memDC.Ellipse(rRunLED);
		DeleteObject(pBackground);
	}
	if (theApp.m_PAM_IO_PORTS.bMonitorLED)
	{
		memDC.SelectObject(pRedBrush);
		memDC.Ellipse(rMonitorLED);
		DeleteObject(pRedBrush);
	}
	else
	{
		memDC.SelectObject(pBackground);
		memDC.Ellipse(rMonitorLED);
		DeleteObject(pBackground);
	}
	if (m_bInterruptEnabledLED)
	{
		memDC.SelectObject(pRedBrush);
		memDC.Ellipse(rInterruptEnableLED);
		DeleteObject(pRedBrush);
	}
	else
	{
		memDC.SelectObject(pBackground);
		memDC.Ellipse(rInterruptEnableLED);
		DeleteObject(pBackground);
	}

//	Now show the MAN1 digits reflecting their current bit patterns.

	for (nDigit = 0; nDigit < 9; ++nDigit)
	{
		rWork.left = nDigitXOrigin[nDigit];
		rWork.top = nDigitYOrigin;
		rWork.right = rWork.left + 25;
		rWork.bottom = rWork.top + 41;
		memDC.SelectObject(pBackground);
		memDC.SelectStockObject(NULL_PEN);
		memDC.Rectangle(rWork + CRect(4,4,4,4));	//	blank the digit.
		DeleteObject(pBackground);
		rSegment.left = rWork.left + rSegA.left;
		rSegment.right = rWork.left + rSegA.right;
		rSegment.top = rWork.top + rSegA.top;
		rSegment.bottom = rWork.top + rSegA.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_A))
		{
			memDC.SelectObject(pRedPen);
			memDC.MoveTo(rSegment.left + 1, rSegment.top + 1);
			memDC.LineTo(rSegment.right - 1, rSegment.top + 1);
			DeleteObject(pRedPen);
		}
		rSegment.left = rWork.left + rSegB.left;
		rSegment.right = rWork.left + rSegB.right;
		rSegment.top = rWork.top + rSegB.top;
		rSegment.bottom = rWork.top + rSegB.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_B))
		{
			memDC.SelectObject(pRedPen);
			memDC.MoveTo(rSegment.left + 1, rSegment.bottom);
			memDC.LineTo(rSegment.right - 1, rSegment.top);
			DeleteObject(pRedPen);
		}
		rSegment.left = rWork.left + rSegC.left;
		rSegment.right = rWork.left + rSegC.right;
		rSegment.top = rWork.top + rSegC.top;
		rSegment.bottom = rWork.top + rSegC.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_C))
		{
			memDC.SelectObject(pRedPen);
			memDC.MoveTo(rSegment.left + 1, rSegment.bottom);
			memDC.LineTo(rSegment.right - 1, rSegment.top);
			DeleteObject(pRedPen);
		}
		rSegment.left = rWork.left + rSegD.left;
		rSegment.right = rWork.left + rSegD.right;
		rSegment.top = rWork.top + rSegD.top;
		rSegment.bottom = rWork.top + rSegD.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_D))
		{
			memDC.SelectObject(pRedPen);
			memDC.MoveTo(rSegment.left + 1, rSegment.top + 1);
			memDC.LineTo(rSegment.right - 1, rSegment.top + 1);
			DeleteObject(pRedPen);
		}
		rSegment.left = rWork.left + rSegE.left;
		rSegment.right = rWork.left + rSegE.right;
		rSegment.top = rWork.top + rSegE.top;
		rSegment.bottom = rWork.top + rSegE.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_E))
		{
			memDC.SelectObject(pRedPen);
			memDC.MoveTo(rSegment.left + 1, rSegment.bottom);
			memDC.LineTo(rSegment.right - 1, rSegment.top);
			DeleteObject(pRedPen);
		}
		rSegment.left = rWork.left + rSegF.left;
		rSegment.right = rWork.left + rSegF.right;
		rSegment.top = rWork.top + rSegF.top;
		rSegment.bottom = rWork.top + rSegF.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_F))
		{
			memDC.SelectObject(pRedPen);
			memDC.MoveTo(rSegment.left + 1, rSegment.bottom);
			memDC.LineTo(rSegment.right - 1, rSegment.top);
			DeleteObject(pRedPen);
		}
		rSegment.left = rWork.left + rSegG.left;
		rSegment.right = rWork.left + rSegG.right;
		rSegment.top = rWork.top + rSegG.top;
		rSegment.bottom = rWork.top + rSegG.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_G))
		{
			memDC.SelectObject(pRedPen);
			memDC.MoveTo(rSegment.left + 1, rSegment.top + 1);
			memDC.LineTo(rSegment.right - 1, rSegment.top + 1);
			DeleteObject(pRedPen);
		}
		rSegment.left = rWork.left + rSegP.left;
		rSegment.right = rWork.left + rSegP.right;
		rSegment.top = rWork.top + rSegP.top;
		rSegment.bottom = rWork.top + rSegP.bottom;
		if (!(theApp.m_PAM_IO_PORTS.nMAN1[nDigit] & PANEL_SEGMENT_P))
		{
			memDC.SelectObject(pRedBrush);
			memDC.SelectObject(pRedPen);
			memDC.Ellipse(rSegment);
			DeleteObject(pRedPen);
			DeleteObject(pRedBrush);
		}
	}

//	Give the display window just a touch of outlining so it doesn't blend into
//	the background too badly.

	memDC.SelectStockObject(NULL_BRUSH);
	memDC.SelectStockObject(BLACK_PEN);
	memDC.Rectangle(rWnd);
	rWnd += CRect(-1,-1,-1,-1);
	memDC.Rectangle(rWnd);
	rWnd += CRect(-1,-1,-1,-1);
	memDC.Rectangle(rWnd);
	memDC.RestoreDC(nDCIdent);
	rWnd += CRect(2,2,2,2);
	dc.BitBlt(rWnd.left,
						rWnd.top,
						rWnd.Width(),
						rWnd.Height(),
						&memDC,
						0,
						0,
						SRCCOPY);
	memDC.SelectObject(pBmpOld);
	delete pRedBrush;
	delete pRedPen;
	delete pBackground;
}


BOOL Display::OnEraseBkgnd(CDC* pDC) 
{
	CRect			rWnd;
	CDC 			memDC;
	CBitmap *		pBmpOld;

	GetClientRect(&rWnd);
	memDC.CreateCompatibleDC(pDC);
	pBmpOld = memDC.SelectObject(&m_bmpBackground);
	pDC -> BitBlt(0, 0, rWnd.right - rWnd.left, rWnd.bottom - rWnd.top,
				&memDC, 0, 0, SRCCOPY);
	memDC.SelectObject(pBmpOld);
	return CWnd::OnEraseBkgnd(pDC);
}
