/************************************************************************
 *																		*
 *	Copyright (c) 2001 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"
#include "PanelButton.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PanelButton

PanelButton::PanelButton()
{
	m_bDepressed = FALSE;
	m_pSignal = NULL;
}

PanelButton::~PanelButton()
{
}


BEGIN_MESSAGE_MAP(PanelButton, CButton)
	//{{AFX_MSG_MAP(PanelButton)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_WM_LBUTTONUP()
	ON_WM_ENABLE()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PanelButton message handlers



void	PanelButton::AutoLoad(UINT	nId, CWnd* pParent)
{
	CRect	rect;
	CWnd *	pWnd;
	CString	strLabel;

	pWnd = (((CDialog *) pParent) -> GetDlgItem(nId));
	pWnd -> GetClientRect(&rect);
	pWnd -> GetWindowText(strLabel);
	pWnd -> ShowWindow(SW_HIDE);
	if(Create(NULL, "Panel_Button", WS_VISIBLE | WS_CHILD | WS_TABSTOP, rect, pParent, nId, NULL))
	{
		if (m_bmUp.LoadBitmap(strLabel + "U") != 0)
		{
			if (m_bmFocus.LoadBitmap(strLabel + "F") == 0)
				m_bmFocus.LoadBitmap(strLabel + "U");
			if (m_bmDown.LoadBitmap(strLabel + "D") == 0)
				m_bmDown.LoadBitmap(strLabel + "U");
			if (m_bmDisabled.LoadBitmap(strLabel + "X") == 0)
				m_bmDisabled.LoadBitmap(strLabel + "U");
		}
	}
}

void PanelButton::OnPaint() 
{
	CPaintDC	dc(this); // device context for painting

	DrawButton(&dc);
}


void PanelButton::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (IsWindowEnabled())
	{
		m_bDepressed = TRUE;
		SetCapture();
		if (m_pSignal != NULL)
			m_pSignal(m_bDepressed);
		SetFocus();
		RedrawWindow();
	}
	CButton::OnLButtonDown(nFlags, point);
}


void PanelButton::OnLButtonUp(UINT nFlags, CPoint point) 
{
	BOOL	bClick = m_bDepressed;
	if (IsWindowEnabled())
	{
		ReleaseCapture();
		m_bDepressed = FALSE;
		if (m_pSignal != NULL)
			m_pSignal(m_bDepressed);
		RedrawWindow();
		if (bClick)
		{
			m_pParent -> PostMessage(WM_COMMAND,
										(WPARAM) ((BN_CLICKED << 16) | m_nIDD),
										(LPARAM) m_hWnd);
		}
	}
	CButton::OnLButtonUp(nFlags, point);
}

void PanelButton::OnSetFocus(CWnd* pOldWnd) 
{
	CButton::OnSetFocus(pOldWnd);
	RedrawWindow();
}

void PanelButton::OnKillFocus(CWnd* pNewWnd) 
{
	CButton::OnKillFocus(pNewWnd);
	RedrawWindow();
}

void PanelButton::OnEnable(BOOL bEnable) 
{
	CButton::OnEnable(bEnable);
	RedrawWindow();
}

BOOL PanelButton::OnEraseBkgnd(CDC* pDC) 
{
	DrawButton(pDC);
//	return CButton::OnEraseBkgnd(pDC);
	return TRUE;
}


void	PanelButton :: DrawButton(CDC * pDC)
{
	CDC 			memDC;
	CBitmap *		pBmpOld;
	CRect			rWnd;
	int				nOldDC;
	BITMAP			bmStruct;

	nOldDC = pDC -> SaveDC();
	GetClientRect(&rWnd);
	memDC.CreateCompatibleDC(pDC);
	if (m_bDepressed)
	{
		pBmpOld = memDC.SelectObject(&m_bmDown);
		m_bmDown.GetBitmap(&bmStruct);
		pDC -> StretchBlt(rWnd.left,
							rWnd.top,
							rWnd.Width(),
							rWnd.Height(),
							&memDC,
							0,
							0,
							bmStruct.bmWidth,
							bmStruct.bmHeight,
							SRCCOPY);
	}
	else if (GetFocus() == this)
	{
		pBmpOld = memDC.SelectObject(&m_bmFocus);
		m_bmFocus.GetBitmap(&bmStruct);
		pDC -> StretchBlt(rWnd.left,
							rWnd.top,
							rWnd.Width(),
							rWnd.Height(),
							&memDC,
							0,
							0,
							bmStruct.bmWidth,
							bmStruct.bmHeight,
							SRCCOPY);
	}
	else if (!IsWindowEnabled())
	{
		pBmpOld = memDC.SelectObject(&m_bmDisabled);
		m_bmDisabled.GetBitmap(&bmStruct);
		pDC -> StretchBlt(rWnd.left,
							rWnd.top,
							rWnd.Width(),
							rWnd.Height(),
							&memDC,
							0,
							0,
							bmStruct.bmWidth,
							bmStruct.bmHeight,
							SRCCOPY);
	}
	else
	{
		pBmpOld = memDC.SelectObject(&m_bmUp);
		m_bmUp.GetBitmap(&bmStruct);
		pDC -> StretchBlt(rWnd.left,
							rWnd.top,
							rWnd.Width(),
							rWnd.Height(),
							&memDC,
							0,
							0,
							bmStruct.bmWidth,
							bmStruct.bmHeight,
							SRCCOPY);
	}
	memDC.SelectObject(pBmpOld);
	pDC -> RestoreDC(nOldDC);
}

BOOL PanelButton::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	m_pParent = pParentWnd;
	m_nIDD = nID;
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}
