/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"

#define	EXTERN extern

#include "virtual_h8.h"
#include "globals.h"
#include "promload.h"

static const int	BAD_NUMBER=1;
static const int	GOOD_NUMBER=0;

static enum	RADIX {OCTAL, DECAMAL, HEX };

static int	ParseAsNumber(char * szToken,
							enum RADIX eRadix,
							int nLength,
							int * nValue);

int	PromLoader(LPCSTR szFilename)
{
	FILE *		ifp;
	enum	{
				GROUND,
				RADIX,
				POSTRADIX,
				ORG,
				POSTORG,
				DATA,
				POSTDATA,
				ENDFOUND,
				ERRORSTATE
			}	eState;
	char *		szKeywords[] = {
								"RADIX",
								"OCT",
								"DEC",
								"HEX",
								"ORG",
								"DATA",
								"END"
							  };
	int			nLimit = sizeof(szKeywords) / sizeof(char *);
	int			nI;
	char		szBuffer[256];
	enum RADIX	eRadix;
	char *		pToken;
	char *		pParse;
	CString		strErr;
	int			nAddress = 0;
	int			nData;



//	Load contents of szFilename (a series of octal, hex or decimal values)
//	into pMemory, clearing subsequent pMemory to a total of nCount bytes.

//	Format of file:
//
//	RADIX {DEC | OCT | HEX}
//	ORG <origin_address>
//	DATA <byte> <byte>...
//	[DATA <byte> <byte>...]...
//	[ORG <origin_address>
//	 DATA <byte> <byte>...
//	 [DATA <byte> <byte>...]...
//	]...
//	END
//
//	where	<byte> is			the byte value expressed as a decimal number, a
//								3-digit octal number or a two-digit hexadecimal
//								number, depending on the radix statement;
//			<origin_address> is	the starting address (relative to pMemory) for
//								the subsequent block of data.  This value may be
//								a decimal number, a six-digit "split octal"
//								number or a four-digit hexadecimal number, again
//								depending on the radix statement.
//
//	Notes:
//		1.	There must be one and only one RADIX statement and it must be the
//			first non-blank, non-comment line.
//		2.	There must be one and only one END statement and no non-blank,
//			non-comment lines may follow it.
//		3.	There must be at least one ORG statement and every ORG statement
//			must be followed by at least one DATA statement.
//		4.	Every DATA statement must have at least one <byte> value.
//		5.	Comments are lines which begin with a semicolon character.
//		6.	Lines must be less than 255 bytes long.
//		7.	White space may be either space, horizontal tab or newline.

	ifp = fopen(szFilename, "rb");
	if (ifp == NULL)
	{
		strErr.Format("Unable to open \"%s\" for read.", szFilename);
		AfxMessageBox(strErr);
		return -1;
	}

//	Prepare the state machine.

	eState = GROUND;
	eRadix = OCTAL;
	while (eState != ERRORSTATE)
	{

//	Exit the loop if E.O.F.

		if (feof(ifp)) break;

//	Read a line.  If the first character is ';' continue.

		fgets(szBuffer, 256, ifp);
		if (szBuffer[0] == ';') continue;

//	If the line is blank, continue.

		pParse = szBuffer;

LOOP:

		pToken = strtok(pParse, " \t\r");
		pParse = NULL;
		if ((*pToken == '\n') || (*pToken == '\r')) continue;

//	Extract the first token and set state based on token.  Use state to extract
//	subsequent tokens.  Continue until end is reached.

		for (nI = 0; nI < nLimit; ++nI)
		{
			if (strcmpi(pToken, szKeywords[nI]) == 0) break;
		}
		switch(eState)
		{
		case GROUND:	//	only legal token is RADIX
			if (nI != 0)
			{
				eState = ERRORSTATE;	//	illegal keyword
				strErr.Format("Unexpected keyword in file \"%s\": "
								"Keyword was \"%s\".  \"RADIX\" expected.",
								szFilename, pToken);
			}
			else
			{
				eState = RADIX;
				goto LOOP;
			}
			break;
		case RADIX:	//	only DEC, OCT or HEX legal here.
			if (nI == 1)
			{
				eRadix = OCTAL;
				eState = POSTRADIX;
				goto LOOP;
			}
			else if (nI == 2)
			{
				eRadix = DECAMAL;
				eState = POSTRADIX;
				goto LOOP;
			}
			else if (nI == 3)
			{
				eRadix = HEX;
				eState = POSTRADIX;
				goto LOOP;
			}
			else
			{
				eState = ERRORSTATE;	//	illegal keyword
				strErr.Format("Unexpected keyword in file \"%s\": "
								"Keyword was \"%s\".  "
								"\"DEC\", \"OCT\" or \"HEX\" expected.",
								szFilename, pToken);
			}
			break;
		case POSTRADIX:	//	only ORG legal here.
			if (nI == 4)
			{
				eState = ORG;
				goto LOOP;
			}
			else
			{
				eState = ERRORSTATE;	//	illegal keyword
				strErr.Format("Unexpected token in file \"%s\": "
								"Token was \"%s\".  "
								"\"ORG\" expected.",
								szFilename, pToken);
			}
			break;
		case ORG:		//	<origin_address> expected here.
			if (ParseAsNumber(pToken, eRadix, 2, &nAddress) == BAD_NUMBER)
			{
				eState = ERRORSTATE;	//	illegal keyword
				strErr.Format("Unexpected token in file \"%s\": "
								"Token was \"%s\".  "
								"<origin_address> token expected.",
								szFilename, pToken);
			}
			else
			{
				eState = POSTORG;
				goto LOOP;
			}
			break;
		case POSTORG:	//	only DATA legal here.
			if (nI == 5)
			{
				eState = DATA;
				goto LOOP;
			}
			else
			{
				eState = ERRORSTATE;	//	illegal keyword
				strErr.Format("Unexpected keyword in file \"%s\": "
								"Keyword was \"%s\".  \"DATA\" expected.",
								szFilename, pToken);
			}
			break;
		case DATA:		//	<byte> expected here.
			if (ParseAsNumber(pToken, eRadix, 1, &nData) == BAD_NUMBER)
			{
				eState = ERRORSTATE;	//	illegal keyword
				strErr.Format("Unexpected token in file \"%s\": "
								"Token was \"%s\".  "
								"<byte> token expected.",
								szFilename, pToken);
			}
			else
			{
				eState = POSTDATA;

//	Write the data and increment the address.

				theApp.m_Processor.m_MemMap.LoadProm(nAddress++, (PBYTE)(nData & 0x0ff));
				goto LOOP;
			}
			break;
		case POSTDATA:	//	DATA, ORG, END or <byte> legal here.
			if (nI == 5)
			{
				eState = DATA;
				goto LOOP;
			}
			else if (nI == 4)
			{
				eState = ORG;
				goto LOOP;
			}
			else if (nI == 6)
			{
				eState = ENDFOUND;
			}
			else if (ParseAsNumber(pToken, eRadix, 1, &nData) == GOOD_NUMBER)
			{
				eState = POSTDATA;

//	Write the data and increment the address.

				theApp.m_Processor.m_MemMap.LoadProm(nAddress++, (PBYTE)(nData & 0x0ff));
				goto LOOP;
			}
			else
			{
				eState = ERRORSTATE;	//	illegal keyword
				strErr.Format("Unexpected token in file \"%s\": "
								"Token was \"%s\".  "
								"<byte>, \"ORG\",  \"DATA\" or \"END\" "
								"expected.",
								szFilename, pToken);
			}
			break;
		case ENDFOUND:	//	nothing more to do.
			break;
		default:		//	bad state!
			strErr.Format("Unexpected state value in state machine while "
							"parsing file \"%s\".  State was %d.",
							szFilename, eState);
			eState = ERRORSTATE;
			break;
		}
		if (eState == ENDFOUND) break;
	}
	fclose(ifp);
	if (eState == ERRORSTATE)
	{
		AfxMessageBox("Parser Error -- " + strErr);
	}
	return (eState != ENDFOUND);
}


static int	ParseAsNumber(char * szToken,
							enum RADIX eRadix,
							int nLength,
							int * nValue)
{
	char *	pWork = szToken;
	int		nWork = 0;
	int		nAccumulator = 0;

	switch(eRadix)
	{
	case OCTAL:				//	octal parses differently for addresses (xxx.xxx)
		if (nLength == 1)	//	while data is simply xxx.
		{
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 3) return BAD_NUMBER;
			nAccumulator = nWork;
			++pWork;
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 7) return BAD_NUMBER;
			nAccumulator = nAccumulator * 8 + nWork;
			++pWork;
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 7) return BAD_NUMBER;
			nAccumulator = nAccumulator * 8 + nWork;
			*nValue = nAccumulator;
		}
		else
		{
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 3) return BAD_NUMBER;
			nAccumulator = nWork;
			++pWork;
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 7) return BAD_NUMBER;
			nAccumulator = nAccumulator * 8 + nWork;
			++pWork;
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 7) return BAD_NUMBER;
			nAccumulator = nAccumulator * 8 + nWork;
			++pWork;
			if (*pWork != '.') return BAD_NUMBER;
			++pWork;
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 3) return BAD_NUMBER;
			nAccumulator = nAccumulator * 4 + nWork;
			++pWork;
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 7) return BAD_NUMBER;
			nAccumulator = nAccumulator * 8 + nWork;
			++pWork;
			if (!isdigit(*pWork)) return BAD_NUMBER;
			nWork = *pWork - '0';
			if (nWork > 7) return BAD_NUMBER;
			nAccumulator = nAccumulator * 8 + nWork;
			*nValue = nAccumulator;
		}
		break;
	case DECAMAL:
		if (!isdigit(*pWork)) return BAD_NUMBER;
		nWork = *pWork - '0';
		nAccumulator = nWork;
		++pWork;
		while (isdigit(*pWork))
		{
			nWork = *pWork - '0';
			nAccumulator = nAccumulator * 10 + nWork;
			++pWork;
		}
		if (nLength == 1)
		{
			if (nAccumulator > 255) return BAD_NUMBER;
		}
		else
		{
			if (nAccumulator > 65535L) return BAD_NUMBER;
		}
		*nValue = nAccumulator;
		break;
	case HEX:
		if (!isxdigit(*pWork)) return BAD_NUMBER;
		nWork = tolower(*pWork) - '0';
		if (nWork > 9) nWork -= ('b' - '9');
		nAccumulator = nWork;
		++pWork;
		while (isxdigit(*pWork))
		{
			nWork = tolower(*pWork) - '0';
			if (nWork > 9) nWork -= ('b' - '9');
			nAccumulator = nAccumulator * 16 + nWork;
			++pWork;
		}
		if (nLength == 1)
		{
			if (nAccumulator > 255) return BAD_NUMBER;
		}
		else
		{
			if (nAccumulator > 65535L) return BAD_NUMBER;
		}
		*nValue = nAccumulator;
		break;
	}
	return GOOD_NUMBER;
}
