/************************************************************************
 *																		*
 *	H8 EMULATOR Project --	BUS.H										*
 *																		*
 *	A simulation of the microprocessor bus connections for 8080-type	*
 *	processors															*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#ifndef _BUS_H_
#define _BUS_H_

#ifndef _UART_H_
#include "uart.h"
#endif

class Bus
{

//	Attributes:

public:

//	These objects are for software control and task synchronization:
	
	CRITICAL_SECTION	m_critBus;
	CRITICAL_SECTION	m_critTime;
	HANDLE				m_hTimeRuns;
	BOOL				m_bSuspend;
	BOOL				m_bNewInput;
	BOOL				m_bNewInterruptResponse;

//	These objects are control and data inputs to the processor

	BOOL				m_bINT10;
	BOOL				m_bRTMSignal;
	BOOL				m_bINT20;
	BOOL				m_bINT30;
	BOOL				m_bINT40;
	BOOL				m_bINT50;
	BOOL				m_bINT60;
	BOOL				m_bINT70;
	BOOL				m_bReset;
	char				m_nBusReadData;
	BOOL				m_bInterruptRequest;

//	These objects are output signals from the processor

	BOOL				m_bIORead;
	BOOL				m_bIOWrite;
	BOOL				m_bIntAck;
	char				m_nBusWriteData;
	unsigned short		m_nBusAddress;


//	Processor status signals connected to the bus:

	BOOL				m_bInterruptsEnabled;
	BOOL				m_bRun;
	BOOL				m_bM1;


//	These signals are not "connected":

	BOOL				m_bSOUT;
	BOOL				m_bNMI;
	BOOL				m_bRST5_5;
	BOOL				m_bRST6_5;
	BOOL				m_bRST7_5;
	BOOL				m_bSIN;

//	The Bus object also handles the details of the serial ports:

	UART				m_ConsoleUART;

protected:

private:
	BOOL				m_bH19DTR;
	BOOL				m_bH19OUT1;
	BOOL				m_bH19OUT2;
	BOOL				m_bH19RTS;
	BOOL				m_bH19SendBreak;
	BOOL				m_bH19DCD;
	BOOL				m_bH19bRI;
	BOOL				m_bH19DSR;
	BOOL				m_bH19CTS;
	BOOL				m_bH19CauseBreak;
	int					m_nH19Cursor;		//	cursor determines what byte is
	CStringList			m_cslH19Messages;	//	  next in head string of list.
	HANDLE				m_hH19;				//	handle for I/O pacing scheduler
//	Methods:

public:
					Bus();
					~Bus();
	void			Reset();
	void			SetInterruptRequests();
	void			UpdateSerialIO();
	void			H19MessageIn(CString strText);
	void			H19Break();

protected:

private:

};

#endif	//	#ifndef _BUS_H_
