/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"
#include <process.h>

#define	EXTERN	extern

#include "cpuproc.h"
#include "virtual_h8.h"
#include "virtual_h8Doc.h"
#include "virtual_h8View.h"
#include "globals.h"
#include "processor.h"
#include "pam.h"
#include "logicanalyzer.h"
#include "ioproc.h"
#include <mmsystem.h>

static	void	UpdateBusFromProcessor();
static	void	UpdateProcessorFromBus();
static	void	DoInputOperations();


//	============================================================================
//	This thread manages the processor and IO and communicates between them and
//	the rest of the simulated hardware (except memory) via signals on the bus:

unsigned int __stdcall CPU_Proc(LPVOID /* param */)
{

//	This process is responsible for the interface between the processor chip
//	simulation and the signals on the bus.  Essentially, it transfers from the
//	bus to the processor, single-steps the processor and then transfers to the
//	bus from the processor.  This insulates the processor from having to deal
//	with the inter-thread signalling issues.

	{	//	to ensure every object goes out of scope before _endthreadex() call.

		int				nM1CycleCount;
		int				nTemp;
		CString			strTemp;
		CStatic *		pCPThreadStatusWindow =
							&(((CVirtual_h8View *) theApp.m_pViewWnd) ->
								m_CPThreadStatusWindow);
		theApp.m_nProcessorCycles = 0;
		nM1CycleCount = 0;


//	See if we're suspended, waiting for the display dialog.

		while (theApp.m_Bus.m_bSuspend)
		{
			Sleep(100);
		}
		while(!IsWindow(pCPThreadStatusWindow -> m_hWnd))
			Sleep(100);
		pCPThreadStatusWindow -> SetWindowText("");
		pCPThreadStatusWindow -> ShowWindow(SW_HIDE);
	 	for(;;)
		{

//	We only check the reset and time stops controls periodically.


//	See if we have any reason not to do a processor cycle.
//
//	NOTE: We can't afford to test hShutdown every time through the CPU loop, so
//	it's only being tested if BUS_RESET is asserted.  Since all places in the
//	code which set the shutdown event first assert BUS_RESET, this works well
//	enough and it avoids a significant (about factor of three!) speed penalty.

			if (theApp.m_Bus.m_bReset)
			{

//	We were been told to RESET.  Do it now before doing a machine cycle.
//	If RESET is active, reset the processor (which will cause the reset to be
//	cleared).  Let the processor do a reset and then update the bus state to
//	match what the processor's state is.

				theApp.m_Processor.Reset();
				theApp.m_Bus.m_bReset = FALSE;
				theApp.m_Bus.Reset();
				UpdateBusFromProcessor();
				if (WaitForSingleObject(theApp.m_IPC.hShutdown, 0) != 
					WAIT_TIMEOUT) goto FINISHED;
				pCPThreadStatusWindow -> SetWindowText("");
				pCPThreadStatusWindow -> ShowWindow(SW_HIDE);
				Sleep(100);
				continue;
			}
			else
			{
				Sleep(0);	//	give other thread(s) a chance
			}

//	There are a few operations which we can't afford to do on every M1 cycle
//	because they take too much processor power, so we do them one time in N.

			if (theApp.m_Processor.m_bM1)
				++theApp.m_nProcessorCycles;
			if ((theApp.m_nProcessorCycles % 64) == 0)
			{

//	Update the interrupt requests from I/O devices and update all serial I/O
//	once every 32 cycles.

				theApp.m_Bus.SetInterruptRequests();
				theApp.m_Bus.UpdateSerialIO();
			}
			if ((theApp.m_nProcessorCycles % 256) == 0)
			{

//	The "TimeRuns" handle lets us stop the realtime interrupt and CPU when
//	we must do non-hardware things (like message traffic on the pipes).  From
//	the standpoint of the emulated H-8, when this event is not signalled, time
//	stops.  Test this event every 128 cycles.

				WaitForSingleObject(theApp.m_Bus.m_hTimeRuns, INFINITE);
			}

//	Limit the processor to executing 295,000 M1 cycles per second.  We do this
//	by limiting the processor to 590 M1 cycles between each 2ms interrupt
//	period (whether or not we actually have the real-time interrupt enabled).
//	The 2ms real-time interrupt code will reset theApp.m_nProcessorCycles.
//	Note: on slow PCs, this will have no effect (except to make the code even
//	slower as it attempts to see how fast it's going!); on fast PCs, it will
//	make the H8 emulation run at approximately the same performance as an actual
//	Z80-based H8 system.

			if (theApp.m_nProcessorCycles >= 590) continue;

//	Single-step interrupt (INT20) is asserted two M1 cycles after
//	theApp.m_PAM_IO_PORTS.bI20 is asserted.  It is deasserted two M-1 cycles
//	after !theApp.m_PAM_IO_PORTS.b20 is deasserted.  Since this piece of code is
//	the simplest place to count M-1 cycles, we'll assert and de-assert INT20
//	here if the conditions stated above are met.  While the enable conditions
//	are false, the M-1 count will be kept at zero, like the hardware would.
//	(The above processor-throttle code is not an issue, I'm pretty sure; there
//	are far fewer than 1000 M1 cycles to execute when we single-step.)
//
//	Normally, the 8080 would go on to do the current instruction and discover
//	the interrupt on the next M1 cycle.  The Processor object takes a shortcut,
//	though -- it doesn't do the current instruction fetch and therefore responds
//	to the interrupt request one opcode earlier.  This is harmless, except for
//	the single-step interrupt.  So we make a "wiring change" in the "hardware"
//	for single-step and make the INT20 happen one M1 cycle later than the actual
//	logic would.

			EnterCriticalSection(&theApp.m_Bus.m_critBus); // Capture the bus
			nM1CycleCount += (theApp.m_Bus.m_bM1 ? 1 : 0);
			if (theApp.m_PAM_IO_PORTS.bI20)
			{
				if (!theApp.m_Bus.m_bINT20)
				{
					if (nM1CycleCount >= 3)
					{
						nM1CycleCount = 0;
						theApp.m_Bus.m_bINT20 = TRUE;
					}
				}
				else
				{
					nM1CycleCount = 0;
				}
			}
			else
			{
				if (theApp.m_Bus.m_bINT20)
				{
					if (nM1CycleCount >= 2)
					{
						nM1CycleCount = 0;
						theApp.m_Bus.m_bINT20 = FALSE;
					}
				}
				else
				{
					nM1CycleCount = 0;
				}
			}

//	Transfer the current "input-side" bus state to the processor's "inputs".

			UpdateProcessorFromBus();

#ifdef _DEBUG

			Trace();

#endif

//	We can now exectue an instruction (though the Execute code may detect one
//	of the interrupt signals and have to handle that.

			if ((nTemp = theApp.m_Processor.Execute()) != 0)
			{
				switch(nTemp)
				{
				case ILLEGAL_OP:
					strTemp ="Illegal Opcode Executed";
					break;
				case UNIMPLEMENTED:
					strTemp = "Unimplemented Opcode Executed";
					break;
				case TERMINATE:
					strTemp = "CPU Terminated";
					break;
				case RESET:
					strTemp = "CPU Reset";
					break;
				case ABORT:
					strTemp = "CPU Abort";
					break;
				default:
					strTemp = "Unknown CPU Fault";
					break;
				}
				if (tfp != NULL)
				{
					fprintf(tfp, "%s\n", (LPCSTR) strTemp);
					fflush(tfp);
				}
				Beep(1000,100);

//	Execution is suspended due to illegal op.  Can only get out of this state
//	via RESET or if killed.

				for (;;)
				{
					LeaveCriticalSection(&theApp.m_Bus.m_critBus);
					if (WaitForSingleObject(theApp.m_IPC.hShutdown, 0) != 
						WAIT_TIMEOUT)
					{
						goto FINISHED;
					}
					if (!strTemp.IsEmpty())
					{
						pCPThreadStatusWindow -> SetWindowText(strTemp);
						pCPThreadStatusWindow -> ShowWindow(SW_SHOW);
						strTemp = "";
					}
					Sleep(100);
					EnterCriticalSection(&theApp.m_Bus.m_critBus);
					if (theApp.m_Bus.m_bReset)
					{
						theApp.m_Processor.Reset();
						theApp.m_Bus.Reset();
						UpdateBusFromProcessor();
						LeaveCriticalSection(&theApp.m_Bus.m_critBus);
						pCPThreadStatusWindow -> SetWindowText("");
						pCPThreadStatusWindow -> ShowWindow(SW_HIDE);
						Sleep(100);
						break;
					}
				}
			}
			else
			{

//	Transfer the current processor's output state to the "output-side" bus.

				UpdateBusFromProcessor();
				LeaveCriticalSection(&theApp.m_Bus.m_critBus);
			}
		}

FINISHED:

		EnterCriticalSection(&theApp.m_Bus.m_critBus);
		theApp.m_Bus.m_bRun = FALSE;
		theApp.m_Bus.m_bInterruptsEnabled = FALSE;
		LeaveCriticalSection(&theApp.m_Bus.m_critBus);
		PlaySound(NULL, NULL, SND_ASYNC);	//	ensure sound off.

	}	//	to ensure every object goes out of scope before _endthreadex() call.

	_endthreadex(0);
	return 0;
}


static void UpdateBusFromProcessor()
{

//	Copy the information from the processor's "pins" to the bus "signals".

#if 0	//	these "signals" are not connected:

	theApp.m_Bus.m_bSOUT = theApp.m_Processor.m_bSOUT;

#endif

	theApp.m_Bus.m_bRun = theApp.m_Processor.m_bRun;
	theApp.m_Bus.m_bInterruptsEnabled = theApp.m_Processor.m_bInterruptsEnabled;
	theApp.m_Bus.m_bIORead = theApp.m_Processor.m_bIORead;
	theApp.m_Bus.m_bIOWrite = theApp.m_Processor.m_bIOWrite;
	theApp.m_Bus.m_bIntAck = theApp.m_Processor.m_bIntAck;
	theApp.m_Bus.m_bM1 = theApp.m_Processor.m_bM1;
	if (theApp.m_Processor.m_bIOWrite)
	{
		theApp.m_Bus.m_nBusAddress = theApp.m_Processor.m_nBusAddress;
		theApp.m_Bus.m_nBusWriteData = theApp.m_Processor.m_nBusWriteData;
	}
	else if (theApp.m_Processor.m_bIORead)
	{
		theApp.m_Bus.m_nBusAddress = theApp.m_Processor.m_nBusAddress;
	}

#ifdef _DEBUG

	else
	{

//	If no IO cycle, we only care about the state of these values when tracing.

		theApp.m_Bus.m_nBusWriteData = 0;
		theApp.m_Bus.m_nBusAddress = 0;
	}
	TraceBus(TRUE);

#endif

	if (theApp.m_Bus.m_bIOWrite)
	{

//	I/O Write cycle

		IOWrite(((theApp.m_Bus.m_nBusAddress >> 8) & 0x00FF),
				theApp.m_Bus.m_nBusWriteData);
		theApp.m_Bus.m_bNewInput = FALSE;

//	Handle the speaker (state of speaker can't change without an IOWrite cycle).

		Sound(theApp.m_PAM_IO_PORTS.bSpkr);
	}
}

static void UpdateProcessorFromBus()
{
	DoInputOperations();

#ifdef _DEBUG

	TraceBus(FALSE);
	
#endif

	theApp.m_Processor.m_bIntReq = (theApp.m_Bus.m_bInterruptRequest != 0);

#if 0	//	these "signals" are not connected:

	theApp.m_Processor.m_bNMI =	FALSE;
	theApp.m_Processor.m_bRST5_5 = FALSE;
	theApp.m_Processor.m_bRST6_5 = FALSE;
	theApp.m_Processor.m_bRST7_5 = FALSE;
	theApp.m_Processor.m_bSIN =	FALSE;

#endif

	if (theApp.m_Bus.m_bNewInput)
	{

//	Get the bus data.

		theApp.m_Bus.m_bNewInput = FALSE;
		theApp.m_Processor.m_nBusReadData = theApp.m_Bus.m_nBusReadData;
	}
	else if (theApp.m_Bus.m_bNewInterruptResponse)
	{

//	Get the bus data.

		theApp.m_Bus.m_bNewInterruptResponse = FALSE;
		theApp.m_Processor.m_nBusReadData = theApp.m_Bus.m_nBusReadData;
	}
	return;
}


void	DoInputOperations()
{
	int				nAddr;


//	IREQ, IO Read and IACK response cycles are simulated here.

//	First test the interrupt request lines.

	if(((theApp.m_Bus.m_bINT10 || theApp.m_Bus.m_bRTMSignal)) ||
		(theApp.m_Bus.m_bINT20) ||
		(theApp.m_Bus.m_bINT30) ||
		(theApp.m_Bus.m_bINT40) ||
		(theApp.m_Bus.m_bINT50) ||
		(theApp.m_Bus.m_bINT60) ||
		(theApp.m_Bus.m_bINT70))
	{
		theApp.m_Bus.m_bInterruptRequest = TRUE;

	}
	else
	{
		theApp.m_Bus.m_bInterruptRequest = FALSE;
	}

//	See if we have a processor I/O or IntAck cycle to deal with.  We do I/O
//	first and then IntAck.

	if (theApp.m_Bus.m_bIORead)
	{

//	I/O Read cycle

		nAddr = (theApp.m_Bus.m_nBusAddress >> 8) & 0x00FF;
		theApp.m_Bus.m_nBusReadData = (char)(IORead(nAddr) & 0x00FF);
		theApp.m_Bus.m_bNewInput = TRUE;
	}
	else if (theApp.m_Bus.m_bIntAck)
	{

//	Synthesize a RST instruction based on which of m_bINT10 through m_bINT70 are
//	set.  The lines are prioritized so the lowest-numbered interrupt wins.  We
//	generate the appropriate RST and clear that interrupt signal only.

		if (theApp.m_Bus.m_bINT10 || theApp.m_Bus.m_bRTMSignal)
		{
			theApp.m_Bus.m_nBusReadData = '\317';
		}
		else
		{
			if (theApp.m_Bus.m_bINT20)
			{
				theApp.m_Bus.m_nBusReadData = '\327';
			}
			else
			{
				if (theApp.m_Bus.m_bINT30)
				{
					theApp.m_Bus.m_nBusReadData = '\337';
				}
				else
				{
					if (theApp.m_Bus.m_bINT40)
					{
						theApp.m_Bus.m_nBusReadData = '\347';
					}
					else
					{
						if (theApp.m_Bus.m_bINT50)
						{
							theApp.m_Bus.m_nBusReadData = '\357';
						}
						else
						{
							if (theApp.m_Bus.m_bINT60)
							{
								theApp.m_Bus.m_nBusReadData = '\367';
							}
							else
							{
								if (theApp.m_Bus.m_bINT70)
								{
									theApp.m_Bus.m_nBusReadData = '\377';
								}
							}
						}
					}
				}
			}
		}
		theApp.m_Bus.m_bNewInterruptResponse = TRUE;
		theApp.m_Bus.m_bNewInput = FALSE;
	}
}
