/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"
#include <process.h>

#define	EXTERN	extern

#include "ioproc.h"
#include "virtual_h8.h"
#include "virtual_h8Doc.h"
#include "virtual_h8View.h"
#include "processor.h"
#include "bus.h"
#include "globals.h"
#include "pam.h"
#include "display.h"
#include "logicanalyzer.h"
#include <mmsystem.h>



//
////////////////////////////////////////////////////////////////////////////////




unsigned int __stdcall Timer_Proc(LPVOID /* param */)
{
	{	//	to ensure every object goes out of scope before _endthreadex() call.
		UINT		nTickId;
		HANDLE		hTick;
		POSITION	posP;
		Schedule	stSchedule;
		POSITION	posQ;

		hTick = CreateEvent(NULL, TRUE, FALSE, NULL);

//	Set up the periodic interrupt that is the master clock: this is a
//	one-millisecond (in "H8-time") interrupt that times all real-time-critical
//	cativities in the emulator such as the PAM 2ms interrupt, the UART data
//	rate and the diskette rotation.  Note: this is relativistic time -- it
//	will only match the wall-clock time if TAU is set to be 1.

		nTickId = timeSetEvent(theApp.m_nTau,
								0,
								(LPTIMECALLBACK) hTick,
								0,
								TIME_PERIODIC | TIME_CALLBACK_EVENT_PULSE);

//	Release the emulation threads and assert reset on the bus.

		theApp.m_Bus.m_bSuspend = TRUE;
		theApp.m_Bus.m_bReset = TRUE;

//	This routine handles the "1ms" timer event that controlls the master timing
//	of all real-time emulations.

		while (WaitForSingleObject(theApp.m_IPC.hShutdown, 0) == WAIT_TIMEOUT)
		{
			Sleep(100);	//	before tick enable, only check every 100 ms.
			while ((theApp.m_bTimerTickEnable) &&
				   (WaitForSingleObject(theApp.m_IPC.hShutdown, 0) == 
					WAIT_TIMEOUT))
			{

//	The "TimeRuns" handle lets us stop the realtime interrupt and CPU when
//	we must do non-hardware things (like message traffic on the pipes).  From
//	the standpoint of the emulated H-8, when this event is not signalled, time
//	stops.  While time is stopped, we don't recognize the 2ms tick interrupt.

				WaitForSingleObject(theApp.m_Bus.m_hTimeRuns, INFINITE);

//	Wait on the 1ms tick.  The actual timing of this tick depends on the O/S
//	model(? -- only tried one example of each system listed):
//
//		Operating System					Period (apparent)
//	-----------------------------			-----------------
//	Windows NT 4.0 Workstation					1.03 ms
//	Windows NT 4.0 Server						2.00 ms
//	Windows 2000 Professional					1.08 ms
//	Windows 2000 Server							not tried
//
//	therefore, clock time and H8 time could differ by a large factor.

				WaitForSingleObject(hTick, INFINITE);

//	Increment the master time value.  This counter overflows in about
//	48*TAU days, so don't leave the application running for months. :)

				if ((++theApp.m_nTimeSincePower % 2) == 0)
				{

//	Reset the CPU's M1 cycle counter throttle.

					theApp.m_nProcessorCycles = 0;

					EnterCriticalSection(&theApp.m_Bus.m_critBus);

//	This is the emulation of the 2ms interrupt signal from the front panel
//	hardware.  In the hardware, this timer also resets the MON led and the
//	segment select register.  We let another piece of code handle the UI's
//	reaction to the timer.  Instead, this code will simply generat an INT10
//	interrupt every 2ms if the RTI interrupt enable is on.

					if (theApp.m_PAM_IO_PORTS.bI10)
					{

//	Signal interrupt request INT10.

						theApp.m_Bus.m_bINT10 = TRUE;
					}
					LeaveCriticalSection(&theApp.m_Bus.m_critBus);
				}

//	Walk down the scheduler array and signal any events that have expired.  As
//	events are signalled, remove the expired schedule request from the list.

				EnterCriticalSection(&theApp.m_Bus.m_critTime);
				posP = theApp.m_clSchedules.GetHeadPosition();
				while (posP != NULL)
				{
					posQ = posP;
					stSchedule = theApp.m_clSchedules.GetNext(posP);
					if (stSchedule.nExpirationTime >=
						theApp.m_nTimeSincePower)
					{
						SetEvent(*stSchedule.pEvent);
						theApp.m_clSchedules.RemoveAt(posQ);
					}
				}				
				LeaveCriticalSection(&theApp.m_Bus.m_critTime);
			}
		}
		timeKillEvent(nTickId);
		CloseHandle(hTick);

	}	//	to ensure every object goes out of scope before _endthreadex() call.
	_endthreadex(0);
	return 0;
}



////////////////////////////////////////////////////////////////////////////////
//
//	IORead
//
//	This function emulates the IO Read operation for all I/O ports of the H-8.
					
int	IORead(int nAddr)
{
	int				nData;
	char			cTape;
	bool			bNotImplemented = FALSE;
	CStatic *		g_pIOThreadStatusWindow;

	switch(nAddr)
	{
	case 0350:
	case 0351:
	case 0352:
	case 0353:
	case 0354:
	case 0355:
	case 0356:
	case 0357:

//	Ports 350 - 357 are reserved for the 8250 UART for the H-19.

		nData = theApp.m_Bus.m_ConsoleUART.Read(nAddr & 7);
		break;
	case 0360:

//	PAM keyboard port -- return nKeycode value.

		nData = (char) theApp.m_PAM_IO_PORTS.nKeycode;
		break;
	case 0361:

//	This port is not implemented, but is decoded in the hardware.  Treat as NOP.

		nData = 0;
		break;
	case 0362:

//	This port is the XCON configuration switches.

		nData = theApp.m_XCON_IO_PORTS.ioXCON_IOR_362;
		break;

//	Ports 370 and 371 were for the cassette tape unit on H8-5.  The interface is
//	via an 8251 USART.

	case 0370:	//	data

//	Rx Data.  Return zero if theApp.m_fpTapeIn is NULL, else return byte value or zero
//	if we read past end of file.

		if (!theApp.m_strTapeInPath.IsEmpty() && (theApp.m_fpTapeIn != NULL))
		{
			if (fread(&cTape, 1, 1, theApp.m_fpTapeIn) != 0)
			{
				nData = cTape & 0x00FF;
			}
			else
			{
				nData = 0;
			}
		}
		else nData = 0;
		break;
	case 0371:	//	control

//	Status:
//	7 - DSR: activates (in reality 5 seconds after, in emulator when) TxE set.
//	6 - SYNDET
//	5 - FE
//	4 - OE
//	3 - PE
//	2 - TxE
//	1 - RxRdy
//	0 - TxRdy

		nData = theApp.m_PAM_IO_PORTS.TapePort.nStatus;
		break;

//	Ports 372 and 373 were the console USART ports if the console was connected
//	via the H8-5's serial I/O port.  Not used.

	case 0372:
//		(fall through)
	case 0373:
		nData = 0;
		bNotImplemented = TRUE;
		break;
	case 0174:				//	H17 USART RDE input port -- read USART data
//		(fall through)
	case 0175:				//	H17 USART SWE input port -- read USART status
//								The status flags are: 
//								Bit 0	 - received data is available 
//								Bit 1	 - receiver overrun (not possible)
//								Bit 2	 - receiver parity error (not used)
//								Bit 6	 - fill character transmitted (not used)
//								Bit 7	 - transmitter buffer empty 

//		(fall through)
	case 0176:				//	H17 USART RR input port -- sets search for SYN
//		(fall through)
	case 0177:				//	H17 drive control input port:
//								Bit 0	 - index/sector pulse
//								Bit 1	 - home
//								Bit 2	 - write protect
//								Bit 3	 - Synch char detected
//								Bit 4..7 - unconnected

//	These are the H-17 ports.  Not implemented yet.

		bNotImplemented = TRUE;
		nData = 0;
		break;
	case 0170:
//		(fall through)
	case 0171:

//	These are the H-67 ports.

//		(fall through)
	default:
		bNotImplemented = TRUE;
		nData = 0;
		break;
	}
	if (bNotImplemented)
	{
		if (tfp != NULL)
		{

//	Log the unimplemented input operation and terminate.

			fprintf(tfp, "Input from unimplemented port %s\n",
					(LPCSTR) Octal(nAddr));
			fflush(tfp);
		}
		g_pIOThreadStatusWindow =
			&(((CVirtual_h8View *) theApp.m_pViewWnd) ->
				m_IOThreadStatusWindow);
		if (IsWindow(g_pIOThreadStatusWindow -> m_hWnd))
		{
			g_pIOThreadStatusWindow -> SetWindowText(
				"I/O read attempted from unimplemented port"
													);
			g_pIOThreadStatusWindow -> ShowWindow(SW_SHOW);
		}
		Sleep(5000);
		SetEvent(theApp.m_IPC.hShutdown);
	}
	return nData;
}



////////////////////////////////////////////////////////////////////////////////
//
//	IOWrite
//
//	This function emulates the IO Write operation for all I/O ports of the H-8.

void	IOWrite(int nAddr, int nData)
{
	int				nDigit;
	char			cTape;
	bool			bNotImplemented = FALSE;
	CStringList		cslArgs;
	static BOOL		bInputLocked = FALSE;
	static BOOL		bOutputLocked = FALSE;
	CStatic *		g_pIOThreadStatusWindow;

	switch(nAddr & 0x00FF)
	{
	case 0350:
	case 0351:
	case 0352:
	case 0353:
	case 0354:
	case 0355:
	case 0356:
	case 0357:

//	Ports 350 - 357 are reserved for the 8250 UART for the H-19.

		theApp.m_Bus.m_ConsoleUART.Write(nAddr & 7, (char)(nData & 0x00FF));
		break;
	case 0360:
		theApp.m_PAM_IO_PORTS.ioPAM_IOW_360 = (char)(nData & 0x00FF);

//	Analysis as of 08/09/2000:
//
//	Writing to port 360 requires bit-mapping to individual control wires.
//	The data bus is inverted and clocked into flops.
//
//	For bMonitorLED, a zero clocked from the bus sets the LED on.  (The timer is
//	supposed to reset it, so the code must keep writing the flop at least once
//	every 2 _milliseconds_.)  Conclusion: this bit must be set TRUE.
//
//	For INT10, clocking a zero from the bus sets INT10 for a few hundred
//	_microseconds_.  (The RTM key sets this signal, also.)  Conclusion: this bit
//	must be set TRUE.  Also, whenever a write to port 360 is performed, the flop
//	set by the 2ms timer is reset.  Conclusion: the theApp.m_Bus.m_bINT10 signal must
//	be reset.
//
//	For INT20, clocking a one from the bus sets this signal (which !EI clears).
//	Conclusion: this bit must be set FALSE.
//
//	For bSpkr, clocking a one from the bus sounds the tone.  Conclusion: this
//	bit must be set FALSE.

		theApp.m_PAM_IO_PORTS.bI10 = ((nData & PANEL_INT_10) != 0);
		theApp.m_PAM_IO_PORTS.bI20 = ((~nData & PANEL_INT_20) != 0);
		theApp.m_PAM_IO_PORTS.bSpkr = ((~nData & PANEL_SPKR) != 0);
		theApp.m_PAM_IO_PORTS.bMonitorLED = ((nData & PANEL_MON) != 0);
		theApp.m_PAM_IO_PORTS.bUpdate = TRUE;
		theApp.m_Bus.m_bINT10 = FALSE;
		break;
	case 0361:
		theApp.m_PAM_IO_PORTS.ioPAM_IOW_361 = (char)(nData & 0x00FF);

//	Write to the Display's MAN1 digit "latch" currently selected, if any.

		nDigit = (theApp.m_PAM_IO_PORTS.ioPAM_IOW_360 &
					PANEL_DIGIT_SELECT_MASK) - 1;
		if ((nDigit >= 0) && (nDigit < 9))
		{
			theApp.m_PAM_IO_PORTS.nMAN1[nDigit] =
				theApp.m_PAM_IO_PORTS.ioPAM_IOW_361;
		}
		theApp.m_PAM_IO_PORTS.bUpdate = TRUE;
		break;
	case 0362:

//	This port is the XCON control register.  Set the appropriate XCON control
//	bits.  Also, pass bRomDisable as an argument to the MemoryMap object's
//	SetRomDisable() method once we're sure of the polarity.

		theApp.m_XCON_IO_PORTS.ioXCON_IOW_362 = (char) nData;
		theApp.m_XCON_IO_PORTS.bNmiMask = 
			(((char) nData & XCON_CTRL_NMIMASK) != 0);
		theApp.m_XCON_IO_PORTS.bRomDisable =
			(((char) nData & XCON_CTRL_ROMDISABLE) != 0);
		theApp.m_XCON_IO_PORTS.bSideSelect =
			(((char) nData & XCON_CTRL_SIDE1SEL) != 0);
		theApp.m_Processor.m_MemMap.SetRomDisable((theApp.m_XCON_IO_PORTS.bRomDisable != 0));

//	(May eventually have to deal with the effects of the SideSelect and NmiMask
//	bits, but for now we simply set them.)

		break;

//	Ports 370 and 371 were for the cassette tape unit on H8-5.  The interface is
//	via an 8251 USART.

	case 0370:	//	data

//	Every byte written here is output to tape.

		if (!theApp.m_strTapeOutPath.IsEmpty() && (theApp.m_fpTapeOut != NULL))
		{
			cTape = (char)(nData & 0x00FF);
			fwrite(&cTape, 1, 1, theApp.m_fpTapeOut);
		}
		break;
	case 0371:	//	control
		if (!theApp.m_PAM_IO_PORTS.TapePort.bCommand_State)
		{

//	First write to port 371 after reset is mode instruction; subsequent writes
//	to this port are command instructions.  We need do nothing with the mode
//	command word bits except ensure the device is programmed for async and put
//	the USART into command state.  Mostly, this logic path exists so we can put
//	in code to open OUT.TAPE preparatory to writing a dump file.

			theApp.m_PAM_IO_PORTS.TapePort.bCommand_State = TRUE;
		}
		else
		{

//	Command word:

//	7 - HuntMode
//	6 - Reset 
//	5 - RTS:		Start recorder for read
//	4 - ErrorReset
//	3 - Break
//	2 - Rx Enable
//	1 - DTR
//	0 - Tx Enable:	Start recorder for write

			if (nData & USART_CMD_RESET)
			{
				theApp.m_PAM_IO_PORTS.TapePort.bCommand_State = FALSE;
				theApp.m_PAM_IO_PORTS.TapePort.nStatus = 0;

//	If TAPE.IN or TAPE.OUT is open, close it.

				if (theApp.m_fpTapeIn != NULL)
				{
					fclose(theApp.m_fpTapeIn);
					theApp.m_fpTapeIn = NULL;
				}
				if (theApp.m_fpTapeOut != NULL)
				{
					fclose(theApp.m_fpTapeOut);
					theApp.m_fpTapeOut = NULL;
				}

//	Unlock both cassettes.

				if (bInputLocked)
				{
					cslArgs.RemoveAll();
					cslArgs.AddTail("0");
					cslArgs.AddTail("0");
					theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
					bInputLocked = FALSE;
				}
				if (bOutputLocked)
				{
					cslArgs.RemoveAll();
					cslArgs.AddTail("1");
					cslArgs.AddTail("0");
					theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
					bOutputLocked = FALSE;
				}
			}
			else
			{
				if (nData & USART_CMD_ERROR_RESET)
				{
					theApp.m_PAM_IO_PORTS.TapePort.nStatus &= ~(USART_STATUS_FE |
														USART_STATUS_OE |
														USART_STATUS_PE);
				}
				if (nData & USART_CMD_HUNT)
				{

//	? -- never issued, normally

				}
				if (nData & USART_CMD_RTS)
				{

//	Start read "motor".  Open IN.TAPE for read.  Lock read cassette.

					if (theApp.m_fpTapeIn == NULL)
					{
						if (!theApp.m_strTapeInPath.IsEmpty())
						{
							theApp.m_fpTapeIn = fopen((LPCSTR) theApp.m_strTapeInPath, "rb");
							if (theApp.m_fpTapeIn != NULL)
							{
								if (!bInputLocked) 
								{
									cslArgs.RemoveAll();
									cslArgs.AddTail("0");
									cslArgs.AddTail("1");
									theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
									bInputLocked = TRUE;
								}
							}
						}
					}
					else if (!bInputLocked) 
					{
						AfxMessageBox("theApp.m_fpTapeIn not NULL and "
											"bInputLocked is FALSE!");
					}
				}
				else
				{

//	Stop read "motor".  Close IN.TAPE.  Unlock cassette.

					if (theApp.m_fpTapeIn != NULL)
					{
						fclose(theApp.m_fpTapeIn);
						theApp.m_fpTapeIn = NULL;
					}
					if (bInputLocked)
					{
						cslArgs.RemoveAll();
						cslArgs.AddTail("0");
						cslArgs.AddTail("0");
						theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
						bInputLocked = FALSE;
					}
				}
				if (nData & USART_CMD_DTR)
				{

//	Does nothing.

				}
				if (nData & USART_CMD_BREAK)
				{

//	? -- never issued, normally.

				}
				if (nData & USART_CMD_RX_ENABLE)
				{
					theApp.m_PAM_IO_PORTS.TapePort.nStatus |= USART_STATUS_RXRDY;
				}
				if (nData & USART_CMD_TX_ENABLE)
				{
					theApp.m_PAM_IO_PORTS.TapePort.nStatus |= USART_STATUS_TXRDY |
														USART_STATUS_TXE;

//	Start write "motor".  If a file is open for output, also set the DSR
//	status bit and lock the cassette.

					if (theApp.m_fpTapeOut == NULL)
					{

//	Open TAPE.OUT for ASCII output, overwrite previous contents.

						if (!theApp.m_strTapeOutPath.IsEmpty())
						{
							theApp.m_fpTapeOut = fopen((LPCSTR) theApp.m_strTapeOutPath, "wb");
						}
						if (theApp.m_fpTapeOut != NULL)
						{
							if (!bOutputLocked)
							{
								cslArgs.RemoveAll();
								cslArgs.AddTail("1");
								cslArgs.AddTail("1");
								theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
								bOutputLocked = TRUE;
							}
							theApp.m_PAM_IO_PORTS.TapePort.nStatus |= USART_STATUS_DSR;
						}
						else
						{
							theApp.m_PAM_IO_PORTS.TapePort.nStatus &= ~USART_STATUS_DSR;
						}
					}
				}
				else
				{

//	Stop write "motor".  If file is open for output, also close file.
//	Unlock the cassette.


					theApp.m_PAM_IO_PORTS.TapePort.nStatus &= ~(USART_STATUS_TXRDY |
														USART_STATUS_TXE |
														USART_STATUS_DSR);
					if (theApp.m_fpTapeOut != NULL)
					{
						fclose(theApp.m_fpTapeOut);
						theApp.m_fpTapeOut = NULL;
					}
					if (bOutputLocked)
					{
						cslArgs.RemoveAll();
						cslArgs.AddTail("1");
						cslArgs.AddTail("0");
						theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
						bOutputLocked = FALSE;
					}
				}
			}
		}
		break;

//	Ports 372 and 373 were the console USART ports if the console was connected
//	via the H8-5's serial I/O port.  Not used.

	case 0372:
//		(fall through)
	case 0373:
		bNotImplemented = TRUE;
		break;
	case 0174:				//	H17 USART TDS port -- write USART data
//		(fall through)
	case 0175:				//	H17 USART TFS port -- the fill character
//		(fall through)
	case 0176:				//	H17 USART RSS port -- the sync character
//		(fall through)
	case 0177:				//	H17 drive control output port:
//								Bit 0	 - write gate
//								Bit 1	 - SY0 select
//								Bit 2	 - SY1 select
//								Bit 3	 - SY2 select
//								Bit 4	 - Motor On
//								Bit 5	 - Direction (0 = out)
//								Bit 6	 - Step
//								Bit 7	 - RAM Write Enable
//	Note: all outputs are zero at reset.
//	Write Gate is also ANDED with Write Protect; if Write Protect is asserted,
//	Write Gate is NOT sent to the drive.

//	These are the H-17 ports.  Not yet implemented.

		bNotImplemented = TRUE;
		break;
	case 0170:
//		(fall through)
	case 0171:

//	These are the H-67 ports.  Not yet implemented.

//		(fall through)
	default:
		bNotImplemented = TRUE;
		break;
	}
	if (bNotImplemented)
	{
		if (tfp != NULL)
		{

//	Log the unimplemented output operation and terminate.

			fprintf(tfp, "***Port %s is not implemented.***\n",
					(LPCSTR) Octal(nAddr));
			fflush(tfp);
		}
		g_pIOThreadStatusWindow =
			&(((CVirtual_h8View *) theApp.m_pViewWnd) -> m_IOThreadStatusWindow);
		if (IsWindow(g_pIOThreadStatusWindow -> m_hWnd))
		{
			g_pIOThreadStatusWindow -> SetWindowText(
				"I/O write attempted to unimplemented port"
													);
			g_pIOThreadStatusWindow -> ShowWindow(SW_SHOW);
		}
		Sleep(5000);
		SetEvent(theApp.m_IPC.hShutdown);
	}
	return;
}


void	Sound(BOOL bState)
{
	static BOOL	bOldState = FALSE;

//	This is the code to emulate the NAND gate and speaker connected to the 1000
//	Hz squarewave through the reed switch.  The reed switch was a non-standard
//	modification I made to the H-8 to allow a program to play music through the
//	front-panel speaker.  When a magnet was placed between the 'H' and 'K' of
//	the Heathkit logo, the reed switch was opened and the 1000 Hz tone became
//	just a click whenever the speaker control bit changed state.  You could then
//	have timing loops that did I/O to this bit and made audio tones.  The code
//	below emulates that feature.  WARNING: because the exact CPU timing is not
//	being simulated, the chances are H8MUSIC.ABS will *not* result in music!

	if (!bOldState && bState)
	{

//	Asynchronous 1000 Hz tone On

		if (!theApp.m_PAM_IO_PORTS.m_bReedSwitch)
		{

//	Without the magnet, we play the tone until told to stop.

			if (theApp.m_hSound)
			{
				PlaySound(MAKEINTRESOURCE(IDR_1KHZ), theApp.m_hInstance,
							SND_ASYNC | SND_LOOP | SND_RESOURCE);
			}
		}
		else
		{

//	With the magnet in place, we play a "click" -- one sample.

			if (theApp.m_hSound)
			{
				PlaySound(MAKEINTRESOURCE(IDR_1KHZ), theApp.m_hInstance,
							SND_ASYNC | SND_RESOURCE);
			}
		}
	}
	else if (bOldState && !bState)
	{

//	Asynchronous 1000 Hz tone Off


		if (!theApp.m_PAM_IO_PORTS.m_bReedSwitch)
		{

//	Without the magnet, we stop playing the tone.

			PlaySound(NULL, NULL, SND_ASYNC);
		}
		else
		{

//	With the magnet in place, we play a "click" -- one sample.

			if (theApp.m_hSound)
			{
				PlaySound(MAKEINTRESOURCE(IDR_1KHZ), theApp.m_hInstance,
							SND_ASYNC | SND_RESOURCE);
			}
		}
	}
	bOldState = bState;
}
