/************************************************************************
 *																		*
 *	Copyright (c) 2001 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/

#include "stdafx.h"

#define	EXTERN extern

#include "virtual_h8.h"
#include "globals.h"
#include "msgs.h"
#include "ipc.h"
#include "lasterrortext.h"

	IPC::IPC()
{
	int		nI;

//	Initialize the global table of pipe names.  This is a little convoluted so
//	that I only have to specify the size of the m_szNames array (in h8ctrl.h)
//	and the sizes of everything else that depends on the number of pipes we have
//	are automatically defined as well.

	strcpy(m_szNames[0], CASSETTE_RX);
	strcpy(m_szNames[1], CASSETTE_TX);
	strcpy(m_szNames[IPC_ARRAY_SIZE - 1], "");

//	Zero out the pipe control structures and message buffers.

	for (nI = 0; nI < (IPC_ARRAY_SIZE - 1); ++nI)
	{
		memset(&stPipes[nI], 0, sizeof(PIPE_CONTROL));
	}
	for (nI = 0; nI < (IPC_ARRAY_SIZE - 1) / 2; ++nI)
	{
		m_cslMessages[nI].RemoveAll();
	}

//	Set all event handles to NULL

	hKillPipeProc = NULL;
	hPipeProc = NULL;
	hShutdown = NULL;

//	Initialize the m_critical section protecting the data buffers.

	InitializeCriticalSection(&m_critMessageBuffers);
}


	IPC::~IPC()
{
	int nI;

	SetEvent(hKillPipeProc);
	theApp.m_Bus.m_bReset = TRUE;
	SetEvent(hShutdown);
	Sleep(1000);
	CloseHandle(hKillPipeProc);
	CloseHandle(hPipeProc);
	CloseHandle(hShutdown);
	for (nI = 0; nI < (IPC_ARRAY_SIZE - 1); ++nI)
	{
		if (stPipes[nI].overControl.hEvent != NULL)
		{
			CloseHandle(stPipes[nI].overControl.hEvent);
			stPipes[nI].overControl.hEvent = NULL;
		}
		if (stPipes[nI].hPipe != NULL)
		{
			CloseHandle(stPipes[nI].hPipe);
			stPipes[nI].hPipe = NULL;
		}
	}
	for (nI = 0; nI < (IPC_ARRAY_SIZE - 1) / 2; ++nI)
	{
		m_cslMessages[nI].RemoveAll();
	}
	DeleteCriticalSection(&m_critMessageBuffers);
}

BOOL	IPC :: CreatePipes(CString & strErrorMessage)
{
	strErrorMessage = "";
	
//	Create the shutdown event in unsignalled state.

	if ((hShutdown =
			CreateEvent(NULL, TRUE, FALSE, NULL)) == NULL)
	{

//	Could not create the event.

		strErrorMessage = "Program error -- could not create the event "
							"for signalling shutdown request detected by "
							"pipe read thread.";
	}
	if (!strErrorMessage.IsEmpty()) return FALSE;
	
//	Configure and create the pipes.

	for (int nPipeNum = 0; m_szNames[nPipeNum][0] != 0; ++nPipeNum)
	{
		stPipes[nPipeNum].dwBufferLength = 0;
		stPipes[nPipeNum].dwXferCount = 0;
		stPipes[nPipeNum].ePipeMode = PIPEMODE_UNDEFINED;
		stPipes[nPipeNum].ePipeState = PIPESTATE_CLOSED;
		memset(&(stPipes[nPipeNum].overControl), 0, sizeof(OVERLAPPED));
		stPipes[nPipeNum].overControl.hEvent = INVALID_HANDLE_VALUE;
		stPipes[nPipeNum].hPipe = INVALID_HANDLE_VALUE;

//	Create manually-reset Event signalled and assign it to pipe.

		if ((stPipes[nPipeNum].overControl.hEvent =
				CreateEvent (NULL, TRUE, TRUE, NULL)) == NULL )
		{
			strErrorMessage.Format("Unable to create an event for "
									"overlapped %s pipe named %s.",
									(nPipeNum % 2) ?
										"read from" : "write to",
									m_szNames[nPipeNum]);
			return FALSE;
		}
		if (!ConnectPipe(nPipeNum, strErrorMessage)) return FALSE;
	}

//	Create auto-reset Event in unsignalled state.  We signal this event if
//	we need to terminate the read pipe thread.

	if ((hKillPipeProc = CreateEvent(NULL, TRUE, FALSE, NULL)) == NULL)
	{

//	Could not create the event.

		strErrorMessage = "Program error -- could not create the event "
							"for signalling termination request to "
							"pipe read thread.";
		return FALSE;
	}
	return TRUE;
}


BOOL	IPC :: DisconnectPipe(int nPipe, CString & strErrorMessage)
{
	strErrorMessage = "";

	if ((nPipe < 0) || (nPipe >= (IPC_ARRAY_SIZE - 1)))
	{
		strErrorMessage = "Program error: invalid pipe number.";
		return FALSE;
	}
	if ((nPipe % 2) == 0)
	{

//	Disconnect an RX pipe and flush its message buffer.

		DisconnectNamedPipe(stPipes[nPipe].hPipe);
		stPipes[nPipe].ePipeState = PIPESTATE_CLOSED;
		ResetEvent(stPipes[nPipe].overControl.hEvent);
		CloseHandle(stPipes[nPipe].hPipe);
		stPipes[nPipe].hPipe = NULL;
		stPipes[nPipe].dwBufferLength = 0;
		stPipes[nPipe].dwXferCount = 0;
		m_cslMessages[nPipe / 2].RemoveAll();
	}
	else
	{

//	Disconnect a TX pipe.

		DisconnectNamedPipe(stPipes[nPipe].hPipe);
		stPipes[nPipe].ePipeState = PIPESTATE_CLOSED;
		ResetEvent(stPipes[nPipe].overControl.hEvent);
		CloseHandle(stPipes[nPipe].hPipe);
		stPipes[nPipe].hPipe = NULL;
		stPipes[nPipe].dwBufferLength = 0;
		stPipes[nPipe].dwXferCount = 0;
	}
	return TRUE;	//	at the moment, there's no way this code fails.
}


BOOL	IPC :: ConnectPipe(int nPipe, CString & strErrorMessage)
{
	DWORD	nResult;

	strErrorMessage = "";
	
	if ((nPipe < 0) || (nPipe >= (IPC_ARRAY_SIZE - 1)))
	{
		strErrorMessage = "Program error: invalid pipe number.";
		return FALSE;
	}
	if ((nPipe % 2) == 0)
	{

//	Create the RX pipe and set it to wait for connection.
//	An RX pipe is, from our point of view, an outbound pipe.

		stPipes[nPipe].hPipe =
			CreateNamedPipe(m_szNames[nPipe],
							PIPE_ACCESS_OUTBOUND | FILE_FLAG_OVERLAPPED,
							PIPE_TYPE_MESSAGE |
								PIPE_READMODE_MESSAGE |
								PIPE_WAIT,
							1,
							MAX_MESSAGE_LENGTH,
							0,
							PIPE_DEFAULT_TIMEOUT,
							NULL);
		if (stPipes[nPipe].hPipe == INVALID_HANDLE_VALUE)
		{

//	Create failed.

			strErrorMessage.Format("Create of outbound pipe (%s) failed.",
								m_szNames[nPipe]);
			return FALSE;
		}
		stPipes[nPipe].ePipeMode = PIPEMODE_SERVER_WRITE_OVERLAPPED;
		stPipes[nPipe].ePipeState = PIPESTATE_CLOSED;

//	Use ConnectNamedPipe to connect to the pipe.
//	This will probably not complete.

		if (!ConnectNamedPipe(stPipes[nPipe].hPipe,
								&stPipes[nPipe].overControl))
		{
			nResult = GetLastError();
			switch(nResult)
			{
			case ERROR_PIPE_CONNECTED:
				stPipes[nPipe].ePipeMode =
					PIPEMODE_SERVER_WRITE_OVERLAPPED;
				stPipes[nPipe].ePipeState = PIPESTATE_IDLE;
				break;
			case ERROR_IO_PENDING:
				break;
			default:
				strErrorMessage.Format("ConnectNamedPipe failed "
										"on Pipe %s: %s",
										m_szNames[nPipe],
										(LPCSTR) GetLastErrorText(nResult));
				break;
			}
			if (!strErrorMessage.IsEmpty()) return FALSE;
		}
	}
	else
	{
	
//	Create the TX pipe.  This is, from our point of view, an inbound pipe.

		stPipes[nPipe].hPipe =
			CreateNamedPipe(m_szNames[nPipe],
							PIPE_ACCESS_INBOUND | FILE_FLAG_OVERLAPPED,
							PIPE_TYPE_MESSAGE |
								PIPE_READMODE_MESSAGE |
								PIPE_WAIT,
							1,
							0,
							MAX_MESSAGE_LENGTH,
							PIPE_DEFAULT_TIMEOUT,
							NULL);
		if (stPipes[nPipe].hPipe == INVALID_HANDLE_VALUE )
		{

//	Create failed.

			strErrorMessage.Format("Create of inbound pipe (%s) failed.",
								m_szNames[nPipe]);
			return FALSE;
		}
		stPipes[nPipe].ePipeMode = PIPEMODE_SERVER_READ_OVERLAPPED;
		stPipes[nPipe].ePipeState = PIPESTATE_CLOSED;

//	Use ConnectNamedPipe to connect to the pipe.
//	This will probably not complete.

		if (!ConnectNamedPipe(stPipes[nPipe].hPipe,
								&stPipes[nPipe].overControl))
		{
			nResult = GetLastError();
			switch(nResult)
			{
			case ERROR_PIPE_CONNECTED:
				stPipes[nPipe].ePipeMode =
					PIPEMODE_SERVER_READ_OVERLAPPED;
				stPipes[nPipe].ePipeState = PIPESTATE_IDLE;
				theApp.m_bConnected[nPipe / 2] = TRUE;
				break;
			case ERROR_IO_PENDING:
				break;
			default:
				strErrorMessage.Format("ConnectNamedPipe failed "
										"on Pipe %s: %s",
										m_szNames[nPipe],
										(LPCSTR) GetLastErrorText(nResult));
				break;
			}
			if (!strErrorMessage.IsEmpty()) return FALSE;
		}
	}
	return TRUE;
}


void	IPC :: AddMessage(int nConnection, CString strText)
{

//	Append strText to the end of m_cslMessages[nConnection].

	if ((nConnection < 0) || (nConnection >= (IPC_ARRAY_SIZE / 2)))
	{
		AfxMessageBox("Program error -- bad connection number");
	}
	else
	{
		EnterCriticalSection(&m_critMessageBuffers);
		m_cslMessages[nConnection].AddTail(strText);
		LeaveCriticalSection(&m_critMessageBuffers);
	}
}


BOOL	IPC :: GetMessage(int nConnection, CString & strText)
{
	BOOL	bReturn;

//	If m_cslMessages[nConnection] is empty, set strText = "" and return FALSE.
//	Otherwise, remove the first message from m_cslMessages[nConnection] and return
//	it in strText and return TRUE.

	if ((nConnection < 0) || (nConnection >= (IPC_ARRAY_SIZE / 2)))
	{
		strText = "";
		return FALSE;
	}
	EnterCriticalSection(&m_critMessageBuffers);
	if (m_cslMessages[nConnection].IsEmpty())
	{
		strText = "";
		bReturn = FALSE;
	}
	else
	{
		strText = m_cslMessages[nConnection].RemoveHead();
		bReturn = TRUE;
	}
	LeaveCriticalSection(&m_critMessageBuffers);
	return bReturn;
}
