/************************************************************************
 *																		*
 *	H8 EMULATOR Project --	IPC.H										*
 *																		*
 *	Named Pipe-based Interprocess Communication for the Heathkit H-8	*
 *	simulator project													*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 2001 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#ifndef _IPC_H_
#define _IPC_H_

//	The IPC data structures: all the objects for pipe I/O
//		There are N-1 named pipes.  They operate in pairs, the even-numbered
//		pipe is an RX pipe -- we send on that one; the odd-numbered pipe is a TX
//		pipe -- we get messages on that one.  For the pipes we get messages on,
//		we need a cslMessages queue.

#define	IPC_ARRAY_SIZE	3
//	The above definition will eventually be 11, because there will be six
//	"peripherals" attached to the H-8: the cassette tape, the H-19 terminal, the
//	H-17 hard-sectored floppies, the H-37 soft-sectored floppies, the H-14
//	printer and the H-67 hard disk (with its 8" floppy).

class	IPC
{

//	Construction/ Destruction:

public:
		IPC();
		~IPC();


//	Methods:

public:
	BOOL	CreatePipes(CString & strErrorMessage);
	BOOL	DisconnectPipe(int nPipe, CString & strErrorMessage);
	BOOL	ConnectPipe(int nPipe, CString & strErrorMessage);
	void	AddMessage(int nConnection, CString strText);
	BOOL	GetMessage(int nConnection, CString & strText);

protected:
private:

//	Attributes:

public:
	HANDLE				hPipeProc;		//	handle to the read pipe thread
	HANDLE				hKillPipeProc;	//	to signal the read pipe thread to terminate.
	HANDLE				hShutdown;		//	signal from pipe read thread to kill app.
	PIPE_CONTROL		stPipes[IPC_ARRAY_SIZE - 1];
protected:
private:
	char 				m_szNames[IPC_ARRAY_SIZE][MAX_PATH];
	CRITICAL_SECTION	m_critMessageBuffers;
	CStringList			m_cslMessages[(IPC_ARRAY_SIZE - 1) / 2];
};

#endif	//	#ifndef _IPC_H_