/************************************************************************
 *																		*
 *	H8 EMULATOR Project --	LOGICANALYZER.H								*
 *																		*
 *	Simulated logic analyzer (debug tool) for the Virtual H-8 Project	*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 1999 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/
#ifndef	_LOGICANALYZER_H_
#define	_LOGICANALYZER_H_

#include	<afxtempl.h>

#ifndef _PANEL_H_
#include "panel.h"
#endif

#include "resource.h"


#define	DEBUG_TRACE_FLAG			0x0001
#define	DEBUG_DUMP_FLAG				0x0002
#define	DEBUG_TRAP_FLAG				0x0004

//	Global logic-analyzer values:

EXTERN	FILE *	tfp;
EXTERN	BOOL	g_bLACycleTrace;
EXTERN	BOOL	g_bLAIACKCycleTrace;
EXTERN	BOOL	g_bLAIENCycleTrace;
EXTERN	BOOL	g_bLAIOReadTrace;
EXTERN	BOOL	g_bLAIOWriteTrace;
EXTERN	BOOL	g_bLAIRQCycleTrace;
EXTERN	BOOL	g_bLAM1CycleTrace;
EXTERN	BOOL	g_bLAResetTrace;
EXTERN	BOOL	g_bLABranchTrace;
EXTERN	BOOL	g_bLADumpPAMArea;

//	Global functions:

CString			SplitOctal(int n);
CString			Octal(int n);
BOOL			IsBranchOp(int n);
void			Trace();
void			TraceBus(BOOL bReaction);
void			MemoryDump();
CString			DecodeBusInterruptState();


class	TraceRegion
{
public:
	TraceRegion() {}


public:
	unsigned short	nStartAddress;
	unsigned short	nLength;
};

class	DumpRegion
{
public:
	DumpRegion() {}


public:
	unsigned short	nStartAddress;
	unsigned short	nLength;
};

class	BreakRegion
{
public:
	BreakRegion() {nType = 1;}


public:
	unsigned short	nStartAddress;
	unsigned short	nLength;
	unsigned char	nType;
};


//	Logic Analizer class:

/////////////////////////////////////////////////////////////////////////////
// LogicAnalyzer dialog



class LogicAnalyzer : public CDialog
{
// Construction
public:
	LogicAnalyzer(CWnd* pParent = NULL);   // standard constructor

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(LogicAnalyzer)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(LogicAnalyzer)
	virtual BOOL OnInitDialog();
	afx_msg void OnBack();
	afx_msg void OnNext();
	virtual void OnOK();
	afx_msg void OnClearBreakBtn();
	afx_msg void OnClearDumpBtn();
	afx_msg void OnClearTraceBtn();
	afx_msg void OnNewBreakBtn();
	afx_msg void OnNewDumpBtn();
	afx_msg void OnNewTraceBtn();
	afx_msg void OnItemchangedBreakList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnItemchangedDumpList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnItemchangedTraceList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDeleteBreakBtn();
	afx_msg void OnDeleteDumpBtn();
	afx_msg void OnDeleteTraceBtn();
	afx_msg void OnDisableAllBreakBtn();
	afx_msg void OnDisableBreakBtn();
	afx_msg void OnEnableAllBreakBtn();
	afx_msg void OnEnableBreakBtn();
	afx_msg void OnPortQaualRd();
	afx_msg void OnPortQaualWr();
	afx_msg void OnLogIor();
	afx_msg void OnLogIow();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()


//	Methods:

public:

protected:

private:
	void	SetListControlHeadings();
	void	LoadTraceRegionsControl();
	void	LoadDumpRegionsControl();
	void	LoadBreakRegionsControl();
	BOOL	ValidatePage(int nPage);
	void	SelectTraceItem(int nItem);
	void	DeSelectTraceItem(int nItem);
	void	SelectDumpItem(int nItem);
	void	DeSelectDumpItem(int nItem);
	void	SelectBreakItem(int nItem);
	void	DeSelectBreakItem(int nItem);
	CString	DecimalToSplitOctal(int nValue);


//	Attributes:

public:

// Dialog Data
	//{{AFX_DATA(LogicAnalyzer)
	enum { IDD = IDD_LOGIC_ANALYZER };
	CButton	m_IOWriteQualCheck;
	CButton	m_IOReadQualCheck;
	CEdit	m_IOWriteEdit;
	CEdit	m_IOReadEdit;
	CListCtrl	m_TraceList;
	CListCtrl	m_DumpList;
	CListCtrl	m_BreakList;
	CButton	m_FinishBtn;
	CButton	m_NextBtn;
	CButton	m_BackBtn;
	BOOL	m_bLogResets;
	BOOL	m_bDumpPAMWorkArea;
	BOOL	m_bTraceBranches;
	UINT	m_nIOReadPort;
	UINT	m_nIOWritePort;
	BOOL	m_bLogAllCycles;
	BOOL	m_bLogIACK;
	BOOL	m_bLogIntEnable;
	BOOL	m_bLogIORead;
	BOOL	m_bLogIOWrite;
	BOOL	m_bLogIRQ;
	BOOL	m_bLogM1;
	BOOL	m_bQualifyIORad;
	BOOL	m_bQualifyIOWrite;
	//}}AFX_DATA


public:

protected:

private:
	PanelManager							m_PanelManager;
	int										m_nPage;
	CList<TraceRegion *, TraceRegion *>		rgnTrace;
	CList<DumpRegion *, DumpRegion *>		rgnDump;
	CList<BreakRegion *, BreakRegion *>		rgnBreak;
};

#endif	//	#ifndef _LOGICANALYZER_H_
