//	Code fragment:  Methods of Processor class for implementing 8080 opcodes


static enum	REGNUM {
					REGNUM_B = 0,
					REGNUM_C,
					REGNUM_D,
					REGNUM_E,
					REGNUM_H,
					REGNUM_L,
					REGNUM_M,
					REGNUM_A,
					REGNUM_AF,
					REGNUM_BC,
					REGNUM_DE,
					REGNUM_HL,
					REGNUM_SP,
					IMMEDIATE_BYTE,
					INPUT_TO_A_WITH_FLAGS,
					INPUT_TO_M_AND_DECREMENT_ONCE,
					INPUT_TO_M_AND_DECREMENT_MANY,
					INPUT_TO_M_AND_INCREMENT_ONCE,
					INPUT_TO_M_AND_INCREMENT_MANY
					//	todo: add Z-80 register codes here...
					};

//	============================================================================
//
//	Implementation of Opcodes
//

//	----------------------------------------------------------------------------
//	000	NOP

void	Processor :: NOP()
{
}

//	----------------------------------------------------------------------------
//	001	LXI		B,w

void	Processor :: LXI_B()
{
	BC = Fetch2();
}

//	----------------------------------------------------------------------------
//	002	STAX	B

void	Processor :: STAX_B()
{
	m_MemMap.Write(BC, A);
}

//	----------------------------------------------------------------------------
//	003	INX		B

void	Processor :: INX_B()
{
	++BC;
}

//	----------------------------------------------------------------------------
//	004	INR		B

void	Processor :: INR_B()
{
	SetFlagsForINR(++B);
}

//	----------------------------------------------------------------------------
//	005	DCR		B

void	Processor :: DCR_B()
{
	SetFlagsForDCR(--B);
}

//	----------------------------------------------------------------------------
//	006	MVI		B,n

void	Processor :: MVI_B()
{
	B = Fetch();
}

//	----------------------------------------------------------------------------
//	007	RLC

void	Processor :: RLC()
{
	CY = ((A & 0x0080) != 0);
	A = (PBYTE) (A << 1);
	if (CY) A = (PBYTE)(A | 0x01);
}

//	----------------------------------------------------------------------------
//	010	Varient

void	Processor :: VAR_010()
{
	m_nStatusOfVarientOp  = VarientOp(0010);
}

//	----------------------------------------------------------------------------
//	011	DAD		B

void	Processor :: DAD_B()
{
	long		nSum;

	nSum = (((long) HL) & 0x00FFFF) +
			(((long) BC) & 0x00FFFF);
	SetFlagsForDAD(HL, BC, nSum);
	HL = (PWORD) nSum;
}

//	----------------------------------------------------------------------------
//	012	LDAX	B

void	Processor :: LDAX_B()
{
		A = m_MemMap.Read(BC);
}

//	----------------------------------------------------------------------------
//	013	DCX		B

void	Processor :: DCX_B()
{
	--BC;
}

//	----------------------------------------------------------------------------
//	014	INR		C

void	Processor :: INR_C()
{
	SetFlagsForINR(++C);
}

//	----------------------------------------------------------------------------
//	015	DCR		C

void	Processor :: DCR_C()
{
	SetFlagsForDCR(--C);
}

//	----------------------------------------------------------------------------
//	016	MVI		C,n

void	Processor :: MVI_C()
{
	C = Fetch();
}

//	----------------------------------------------------------------------------
//	017	RRC

void	Processor :: RRC()
{
	CY = ((A & 0x01) != 0);
	A = (PBYTE)(0x007f & (A >> 1));
	if (CY) A = (PBYTE)(A | 0x0080);
}

//	----------------------------------------------------------------------------
//	020	Varient

void	Processor :: VAR_020()
{
	m_nStatusOfVarientOp  = VarientOp(0020);
}

//	----------------------------------------------------------------------------
//	021	LXI		D,w

void	Processor :: LXI_D()
{
	DE = Fetch2();
}

//	----------------------------------------------------------------------------
//	022	STAX	D

void	Processor :: STAX_D()
{
	m_MemMap.Write(DE, A);
}

//	----------------------------------------------------------------------------
//	023	INX		D

void	Processor :: INX_D()
{
	++DE;
}

//	----------------------------------------------------------------------------
//	024	INR		D

void	Processor :: INR_D()
{
	SetFlagsForINR(++D);
}

//	----------------------------------------------------------------------------
//	025	DCR		D

void	Processor :: DCR_D()
{
	SetFlagsForDCR(--D);
}

//	----------------------------------------------------------------------------
//	026	MVI		D,n

void	Processor :: MVI_D()
{
	D = Fetch();
}

//	----------------------------------------------------------------------------
//	027	RAL

void	Processor :: RAL()
{
	PFLAG	workC = ((A & 0x0080) != 0);
	A = (PBYTE) (A << 1);
	if (CY) A = (PBYTE)(A | 0x01);
	CY = workC;

}
//	----------------------------------------------------------------------------
//	030	Varient

void	Processor :: VAR_030()
{
	m_nStatusOfVarientOp  = VarientOp(0030);
}

//	----------------------------------------------------------------------------
//	031	DAD		D

void	Processor :: DAD_D()
{
	long		nSum;

	nSum = (((long) HL) & 0x00FFFF) +
			(((long) DE) & 0x00FFFF);
	SetFlagsForDAD(HL, DE, nSum);
	HL = (PWORD) nSum;
}

//	----------------------------------------------------------------------------
//	032	LDAX	D

void	Processor :: LDAX_D()
{
	A = m_MemMap.Read(DE);
}

//	----------------------------------------------------------------------------
//	033	DCX		D

void	Processor :: DCX_D()
{
	--DE;
}

//	----------------------------------------------------------------------------
//	034	INR		E

void	Processor :: INR_E()
{
	SetFlagsForINR(++E);
}

//	----------------------------------------------------------------------------
//	035	DCR		E

void	Processor :: DCR_E()
{
	SetFlagsForDCR(--E);
}

//	----------------------------------------------------------------------------
//	036	MVI		E,n

void	Processor :: MVI_E()
{
	E = Fetch();
}

//	----------------------------------------------------------------------------
//	037	RAR

void	Processor :: RAR()
{
	PFLAG	workC = ((A & 0x01) != 0);
	A = (PBYTE)(0x007f & (A >> 1));
	if (CY) A = (PBYTE) (A | 0x0080);
	CY = workC;
}

//	----------------------------------------------------------------------------
//	040	Varient

void	Processor :: VAR_040()
{
	m_nStatusOfVarientOp  = VarientOp(0040);
}

//	----------------------------------------------------------------------------
//	041	LXI		H,w

void	Processor :: LXI_H()
{
	HL = Fetch2();
}

//	----------------------------------------------------------------------------
//	042	SHLD	w

void	Processor :: SHLD()
{
	PWORD w = Fetch2();
	m_MemMap.Write2(w, HL);
}

//	----------------------------------------------------------------------------
//	043	INX		H

void	Processor :: INX_H()
{
	++HL;
}

//	----------------------------------------------------------------------------
//	044	INR		H

void	Processor :: INR_H()
{
	SetFlagsForINR(++H);
}

//	----------------------------------------------------------------------------
//	045	DCR		H

void	Processor :: DCR_H()
{
	SetFlagsForDCR(--H);
}

//	----------------------------------------------------------------------------
//	046	MVI		H,n

void	Processor :: MVI_H()
{
	H = Fetch();
}

//	----------------------------------------------------------------------------
//	047	DAA

void	Processor :: DAA()
{
	m_nStatusOfVarientOp = VarientOp(0047);
}

//	----------------------------------------------------------------------------
//	050	Varient

void	Processor :: VAR_050()
{
	m_nStatusOfVarientOp  = VarientOp(0050);
}

//	----------------------------------------------------------------------------
//	051	DAD		H

void	Processor :: DAD_H()
{
	long		nSum;

	nSum = (((long) HL) & 0x00FFFF) +
			(((long) HL) & 0x00FFFF);
	SetFlagsForDAD(HL, HL, nSum);
	HL = (PWORD) nSum;
}

//	----------------------------------------------------------------------------
//	052	LHLD	w

void	Processor :: LHLD()
{
	PWORD	w = Fetch2();
	HL = m_MemMap.Read2(w);
}

//	----------------------------------------------------------------------------
//	053	DCX		H

void	Processor :: DCX_H()
{
	--HL;
}

//	----------------------------------------------------------------------------
//	054	INR		L

void	Processor :: INR_L()
{
	SetFlagsForINR(++L);
}

//	----------------------------------------------------------------------------
//	055	DCR		L

void	Processor :: DCR_L()
{
	SetFlagsForDCR(--L);
}

//	----------------------------------------------------------------------------
//	056	MVI		L,n

void	Processor :: MVI_L()
{
	L = Fetch();
}

//	----------------------------------------------------------------------------
//	057	CMA

void	Processor :: CMA()
{
	A = (PBYTE) ~A;
	AC = true;		//	TODO:	is this true for 8080 & 8085?
	AS = true;
}

//	----------------------------------------------------------------------------
//	060	Varient

void	Processor :: VAR_060()
{
	m_nStatusOfVarientOp  = VarientOp(0060);
}

//	----------------------------------------------------------------------------
//	061	LXI		SP,w

void	Processor :: LXI_SP()
{
	SP = Fetch2();
}

//	----------------------------------------------------------------------------
//	062	STA		w

void	Processor :: STA()
{
	PWORD	w = Fetch2();
	m_MemMap.Write(w, A);
}

//	----------------------------------------------------------------------------
//	063	INX		SP

void	Processor :: INX_SP()
{
	++SP;
}

//	----------------------------------------------------------------------------
//	064	INR		M

void	Processor :: INR_M()
{
	PBYTE	cTemp = m_MemMap.Read(HL);

//	Increment the data and write it to the target.

	m_MemMap.Write(HL, ++cTemp);
	SetFlagsForINR(cTemp);
}

//	----------------------------------------------------------------------------
//	065	DCR		M

void	Processor :: DCR_M()
{
	PBYTE	cTemp = m_MemMap.Read(HL);

//	Decrement the data and write it to the target.

	m_MemMap.Write(HL, --cTemp);
	SetFlagsForDCR(cTemp);
}

//	----------------------------------------------------------------------------
//	066	MVI		M,n

void	Processor :: MVI_M()
{
	PBYTE cTemp = Fetch();
	m_MemMap.Write(HL, cTemp);
}

//	----------------------------------------------------------------------------
//	067	STC

void	Processor :: STC()
{
	CY = true;
}

//	----------------------------------------------------------------------------
//	070	Varient

void	Processor :: VAR_070()
{
	m_nStatusOfVarientOp  = VarientOp(0070);
}

//	----------------------------------------------------------------------------
//	071	DAD		SP

void	Processor :: DAD_SP()
{
	long		nSum;

	nSum = (((long) HL) & 0x00FFFF) +
			(((long) SP) & 0x00FFFF);
	SetFlagsForDAD(HL, SP, nSum);
	HL = (PWORD) nSum;
}

//	----------------------------------------------------------------------------
//	072	LDA		w

void	Processor :: LDA()
{
	A = m_MemMap.Read(Fetch2());
}

//	----------------------------------------------------------------------------
//	073	DCX		SP

void	Processor :: DCX_SP()
{
	--SP;
}

//	----------------------------------------------------------------------------
//	074	INR		A

void	Processor :: INR_A()
{
	SetFlagsForINR(++A);
}

//	----------------------------------------------------------------------------
//	075	DCR		A

void	Processor :: DCR_A()
{
	SetFlagsForDCR(--A);
}

//	----------------------------------------------------------------------------
//	076	MVI		A,n

void	Processor :: MVI_A()
{
	A = Fetch();
}
//	----------------------------------------------------------------------------
//	077	CMC

void	Processor :: CMC()
{
	CY = !CY;
}
//	----------------------------------------------------------------------------
//	100 - 165, 167-177	MOV		r,r
//	166	HLT

void	Processor :: MOV_BB()
{
}

void	Processor :: MOV_BC()
{
	B = C;
}

void	Processor :: MOV_BD()
{
	B = D;
}

void	Processor :: MOV_BE()
{
	B = E;
}

void	Processor :: MOV_BH()
{
	B = H;
}

void	Processor :: MOV_BL()
{
	B = L;
}

void	Processor :: MOV_BM()
{
	B = m_MemMap.Read(HL);
}

void	Processor :: MOV_BA()
{
	B = A;
}

void	Processor :: MOV_CB()
{
	C = B;
}

void	Processor :: MOV_CC()
{
}

void	Processor :: MOV_CD()
{
	C = D;
}

void	Processor :: MOV_CE()
{
	C = E;
}

void	Processor :: MOV_CH()
{
	C = H;
}

void	Processor :: MOV_CL()
{
	C = L;
}

void	Processor :: MOV_CM()
{
	C = m_MemMap.Read(HL);
}

void	Processor :: MOV_CA()
{
	C = A;
}

void	Processor :: MOV_DB()
{
	D = B;
}

void	Processor :: MOV_DC()
{
	D = C;
}

void	Processor :: MOV_DD()
{
}

void	Processor :: MOV_DE()
{
	D = E;
}

void	Processor :: MOV_DH()
{
	D = H;
}

void	Processor :: MOV_DL()
{
	D = L;
}

void	Processor :: MOV_DM()
{
	D = m_MemMap.Read(HL);
}

void	Processor :: MOV_DA()
{
	D = A;
}

void	Processor :: MOV_EB()
{
	E = B;
}

void	Processor :: MOV_EC()
{
	E = C;
}

void	Processor :: MOV_ED()
{
	E = D;
}

void	Processor :: MOV_EE()
{
}

void	Processor :: MOV_EH()
{
	E = H;
}

void	Processor :: MOV_EL()
{
	E = L;
}

void	Processor :: MOV_EM()
{
	E = m_MemMap.Read(HL);
}

void	Processor :: MOV_EA()
{
	E = A;
}

void	Processor :: MOV_HB()
{
	H = B;
}

void	Processor :: MOV_HC()
{
	H = C;
}

void	Processor :: MOV_HD()
{
	H = D;
}

void	Processor :: MOV_HE()
{
	H = E;
}

void	Processor :: MOV_HH()
{
}

void	Processor :: MOV_HL()
{
	H = L;
}

void	Processor :: MOV_HM()
{
	H = m_MemMap.Read(HL);
}

void	Processor :: MOV_HA()
{
	H = A;
}

void	Processor :: MOV_LB()
{
	L = B;
}

void	Processor :: MOV_LC()
{
	L = C;
}

void	Processor :: MOV_LD()
{
	L = D;
}

void	Processor :: MOV_LE()
{
	L = E;
}

void	Processor :: MOV_LH()
{
	L = H;
}

void	Processor :: MOV_LL()
{
}

void	Processor :: MOV_LM()
{
	L = m_MemMap.Read(HL);
}

void	Processor :: MOV_LA()
{
	L = A;
}

void	Processor :: MOV_MB()
{
	m_MemMap.Write(HL, B);
}

void	Processor :: MOV_MC()
{
	m_MemMap.Write(HL, C);
}

void	Processor :: MOV_MD()
{
	m_MemMap.Write(HL, D);
}

void	Processor :: MOV_ME()
{
	m_MemMap.Write(HL, E);
}

void	Processor :: MOV_MH()
{
	m_MemMap.Write(HL, H);
}

void	Processor :: MOV_ML()
{
	m_MemMap.Write(HL, L);
}

void	Processor :: HLT()
{
	m_nStatusOfVarientOp = HALTED;
}

void	Processor :: MOV_MA()
{
	m_MemMap.Write(HL, A);
}

void	Processor :: MOV_AB()
{
	A = B;
}

void	Processor :: MOV_AC()
{
	A = C;
}

void	Processor :: MOV_AD()
{
	A = D;
}

void	Processor :: MOV_AE()
{
	A = E;
}

void	Processor :: MOV_AH()
{
	A = H;
}

void	Processor :: MOV_AL()
{
	A = L;
}

void	Processor :: MOV_AM()
{
	A = m_MemMap.Read(HL);
}

void	Processor :: MOV_AA()
{
}

//	----------------------------------------------------------------------------
//	200	ADD B

void	Processor :: ADD_B()
{
	PBYTE	nTemp = A;

	A = (PBYTE)(A + B);
	SetFlagsForAdd(nTemp, B);
}

//	----------------------------------------------------------------------------
//	201	ADD C

void	Processor :: ADD_C()
{
	PBYTE	nTemp = A;

	A = (PBYTE)(A + C);
	SetFlagsForAdd(nTemp, C);
}

//	----------------------------------------------------------------------------
//	202	ADD D

void	Processor :: ADD_D()
{
	PBYTE	nTemp = A;

	A = (PBYTE)(A + D);
	SetFlagsForAdd(nTemp, D);
}

//	----------------------------------------------------------------------------
//	203	ADD E

void	Processor :: ADD_E()
{
	PBYTE	nTemp = A;

	A = (PBYTE)(A + E);
	SetFlagsForAdd(nTemp, E);
}

//	----------------------------------------------------------------------------
//	204	ADD H

void	Processor :: ADD_H()
{
	PBYTE	nTemp = A;

	A = (PBYTE)(A + H);
	SetFlagsForAdd(nTemp, H);
}

//	----------------------------------------------------------------------------
//	205	ADD L

void	Processor :: ADD_L()
{
	PBYTE	nTemp = A;

	A = (PBYTE)(A + L);
	SetFlagsForAdd(nTemp, L);
}

//	----------------------------------------------------------------------------
//	206	ADD M

void	Processor :: ADD_M()
{
	PBYTE	nTemp = A;
	PBYTE	nOp = m_MemMap.Read(HL);

	A = (PBYTE)(A + nOp);
	SetFlagsForAdd(nTemp, nOp);
}

//	----------------------------------------------------------------------------
//	207	ADD A

void	Processor :: ADD_A()
{
	PBYTE	nTemp = A;

	A = (PBYTE)(A + A);
	SetFlagsForAdd(nTemp, nTemp);
}


//	----------------------------------------------------------------------------
//	210	ADC B

void	Processor :: ADC_B()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + B + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, B, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	211	ADC C

void	Processor :: ADC_C()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + C + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, C, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	212	ADC D

void	Processor :: ADC_D()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + D + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, D, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	213	ADC E

void	Processor :: ADC_E()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + E + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, E, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	214	ADC H

void	Processor :: ADC_H()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + H + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, H, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	215	ADC L

void	Processor :: ADC_L()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + L + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, L, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	216	ADC M

void	Processor :: ADC_M()
{
	PBYTE	nBefore = A;
	PBYTE	nOp = m_MemMap.Read(HL);
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + nOp + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, nOp, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	217	ADC A

void	Processor :: ADC_A()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + nBefore + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, nBefore, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	220	SUB B

void	Processor :: SUB_B()
{
	PBYTE	nBefore = A;

	A = (PBYTE)(A - B);
	SetFlagsForSubtract(nBefore, B);
}

//	----------------------------------------------------------------------------
//	221	SUB C

void	Processor :: SUB_C()
{
	PBYTE	nBefore = A;

	A = (PBYTE)(A - C);
	SetFlagsForSubtract(nBefore, C);
}

//	----------------------------------------------------------------------------
//	222	SUB D

void	Processor :: SUB_D()
{
	PBYTE	nBefore = A;

	A = (PBYTE)(A - D);
	SetFlagsForSubtract(nBefore, D);
}

//	----------------------------------------------------------------------------
//	223	SUB E

void	Processor :: SUB_E()
{
	PBYTE	nBefore = A;

	A = (PBYTE)(A - E);
	SetFlagsForSubtract(nBefore, E);
}

//	----------------------------------------------------------------------------
//	224	SUB H

void	Processor :: SUB_H()
{
	PBYTE	nBefore = A;

	A = (PBYTE)(A - H);
	SetFlagsForSubtract(nBefore, H);
}

//	----------------------------------------------------------------------------
//	255	SUB L

void	Processor :: SUB_L()
{
	PBYTE	nBefore = A;

	A = (PBYTE)(A - L);
	SetFlagsForSubtract(nBefore, L);
}

//	----------------------------------------------------------------------------
//	226	SUB M

void	Processor :: SUB_M()
{
	PBYTE	nBefore = A;
	PBYTE	nOp = m_MemMap.Read(HL);

	A = (PBYTE)(A - nOp);
	SetFlagsForSubtract(nBefore, nOp);
}

//	----------------------------------------------------------------------------
//	227	SUB A

void	Processor :: SUB_A()
{
	PBYTE	nBefore = A;
	A = 0;
	SetFlagsForSubtract(nBefore, nBefore);
}

//	----------------------------------------------------------------------------
//	230	SBB B

void	Processor :: SBB_B()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - B - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, B, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	231	SBB C

void	Processor :: SBB_C()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - C - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, C, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	232	SBB D

void	Processor :: SBB_D()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - D - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, D, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	233	SBB E

void	Processor :: SBB_E()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - E - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, E, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	234	SBB H

void	Processor :: SBB_H()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - H - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, H, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	235	SBB L

void	Processor :: SBB_L()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - L - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, L, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	236	SBB M

void	Processor :: SBB_M()
{
	PBYTE nOp = m_MemMap.Read(HL);
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - nOp - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, nOp, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	237	SBB A

void	Processor :: SBB_A()
{
	PBYTE	nBefore = A;
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (0 - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, nBefore, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	240	ANA B

void	Processor :: ANA_B()
{
	A = (PBYTE)(A & B);
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	241	ANA C

void	Processor :: ANA_C()
{
	A = (PBYTE)(A & C);
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	242	ANA D

void	Processor :: ANA_D()
{
	A = (PBYTE)(A & D);
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	243	ANA E

void	Processor :: ANA_E()
{
	A = (PBYTE)(A & E);
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	244	ANA H

void	Processor :: ANA_H()
{
	A = (PBYTE)(A & H);
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	245	ANA L

void	Processor :: ANA_L()
{
	A = (PBYTE)(A & L);
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	246	ANA M

void	Processor :: ANA_M()
{
	A = (PBYTE)(A & m_MemMap.Read(HL));
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	247	ANA A

void	Processor :: ANA_A()
{
//	A = (PBYTE)(A & A);	identity -- just set flags.
	SetFlagsForAND();
}

//	----------------------------------------------------------------------------
//	250	XRA B

void	Processor :: XRA_B()
{
	A = (PBYTE) (A ^ B);
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	251	XRA C

void	Processor :: XRA_C()
{
	A = (PBYTE) (A ^ C);
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	252	XRA D

void	Processor :: XRA_D()
{
	A = (PBYTE) (A ^ D);
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	253	XRA E

void	Processor :: XRA_E()
{
	A = (PBYTE) (A ^ E);
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	254	XRA H

void	Processor :: XRA_H()
{
	A = (PBYTE) (A ^ H);
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	255	XRA L

void	Processor :: XRA_L()
{
	A = (PBYTE) (A ^ L);
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	256	XRA M

void	Processor :: XRA_M()
{
	A = (PBYTE) (A ^ m_MemMap.Read(HL));
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	257	XRA A

void	Processor :: XRA_A()
{
	A = 0;
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	260	ORA B

void	Processor :: ORA_B()
{
	A = (PBYTE) (A | B);
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	261	ORA C

void	Processor :: ORA_C()
{
	A = (PBYTE) (A | C);
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	262	ORA D

void	Processor :: ORA_D()
{
	A = (PBYTE) (A | D);
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	263	ORA E

void	Processor :: ORA_E()
{
	A = (PBYTE) (A | E);
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	264	ORA H

void	Processor :: ORA_H()
{
	A = (PBYTE) (A | H);
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	265	ORA L

void	Processor :: ORA_L()
{
	A = (PBYTE) (A | L);
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	266	ORA M

void	Processor :: ORA_M()
{
	A = (PBYTE) (A | m_MemMap.Read(HL));
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	267	ORA A

void	Processor :: ORA_A()
{
//	A = (PBYTE) (A | A);	identity -- just set flags.
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	270	CMP B

void	Processor :: CMP_B()
{
	SetFlagsForCompare(B);
}

//	----------------------------------------------------------------------------
//	271	CMP C

void	Processor :: CMP_C()
{
	SetFlagsForCompare(C);
}

//	----------------------------------------------------------------------------
//	272	CMP D

void	Processor :: CMP_D()
{
	SetFlagsForCompare(D);
}

//	----------------------------------------------------------------------------
//	273	CMP E

void	Processor :: CMP_E()
{
	SetFlagsForCompare(E);
}

//	----------------------------------------------------------------------------
//	274	CMP H

void	Processor :: CMP_H()
{
	SetFlagsForCompare(H);
}

//	----------------------------------------------------------------------------
//	255	CMP L

void	Processor :: CMP_L()
{
	SetFlagsForCompare(L);
}

//	----------------------------------------------------------------------------
//	276	CMP M

void	Processor :: CMP_M()
{
	SetFlagsForCompare(m_MemMap.Read(HL));
}

//	----------------------------------------------------------------------------
//	277	CMP A

void	Processor :: CMP_A()
{
	SetFlagsForCompare(A);
}

//	----------------------------------------------------------------------------
//	300	RNZ

void	Processor :: RNZ()
{
	if (!Z)
	{
		PC = m_MemMap.Read2(SP);
		SP += 2;
	}
}

//	----------------------------------------------------------------------------
//	301	POP	B

void	Processor :: POP_B()
{
	BC = m_MemMap.Read2(SP);
	SP += 2;
}

//	----------------------------------------------------------------------------
//	302	JNZ	w

void	Processor :: JNZ()
{
	PWORD	w = Fetch2();

	if (!Z)
	{
		PC = w;
	}
}

//	----------------------------------------------------------------------------
//	303	JMP	w

void	Processor :: JMP()
{
	PWORD	w = Fetch2();
	PC = w;
}

//	----------------------------------------------------------------------------
//	304	CNZ	w

void	Processor :: CNZ()
{
	PWORD	w = (PWORD) (PC + 2);

	if (!Z)
	{
		SP -= 2;
		m_MemMap.Write2(SP, w);
		w = Fetch2();
	}
	PC = w;
}

//	----------------------------------------------------------------------------
//	305	PUSH B

void	Processor :: PUSH_B()
{
	SP -= 2;
	m_MemMap.Write2(SP, BC);
}

//	----------------------------------------------------------------------------
//	306	ADI	n

void	Processor :: ADI()
{
	PBYTE	nBefore = A;
	PBYTE	nOp = m_MemMap.Read(PC++);

	A = (PBYTE)(A + nOp);
	SetFlagsForAdd(nBefore, nOp);
}

//	----------------------------------------------------------------------------
//	307	RST	0

void	Processor :: RST_0()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0000;
}

//	----------------------------------------------------------------------------
//	310	RZ

void	Processor :: RZ()
{
	if (Z)
	{
		PC = m_MemMap.Read2(SP);
		SP += 2;
	}
}

//	----------------------------------------------------------------------------
//	311	RET

void	Processor :: RET()
{
		PC = m_MemMap.Read2(SP);
		SP += 2;
}

//	----------------------------------------------------------------------------
//	312	JZ	w

void	Processor :: JZ()
{
	PWORD	w = Fetch2();

	if (Z)
	{
		PC = w;
	}
}

//	----------------------------------------------------------------------------
//	313	Varient

void	Processor :: VAR_313()
{
	m_nStatusOfVarientOp  = VarientOp(0313);
}

//	----------------------------------------------------------------------------
//	314	CZ	w

void	Processor :: CZ()
{
	PWORD	w = (PWORD) (PC + 2);

	if (Z)
	{
		SP -= 2;
		m_MemMap.Write2(SP, w);
		w = Fetch2();
	}
	PC = w;
}

//	----------------------------------------------------------------------------
//	315	CALL w

void	Processor :: CALL()
{
	PWORD	w = (PWORD) (PC + 2);

	SP -= 2;
	m_MemMap.Write2(SP, w);
	PC = Fetch2();
}

//	----------------------------------------------------------------------------
//	316	ACI	n

void	Processor :: ACI()
{
	PBYTE	nBefore = A;
	PBYTE	nOp = m_MemMap.Read(PC++);
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (nBefore + nOp + (bCY_OLD ? 1 : 0));
	SetFlagsForADC(nBefore, nOp, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	317	RST	1

void	Processor :: RST_1()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0008;
}

//	----------------------------------------------------------------------------
//	320	RNC

void	Processor :: RNC()
{
	if (!CY)
	{
		PC = m_MemMap.Read2(SP);
		SP += 2;
	}
}

//	----------------------------------------------------------------------------
//	321	POP	D

void	Processor :: POP_D()
{
		DE = m_MemMap.Read2(SP);
		SP += 2;
}

//	----------------------------------------------------------------------------
//	322	JNC w

void	Processor :: JNC()
{
	PWORD	w = Fetch2();

	if (!CY)
	{
		PC = w;
	}
}

//	----------------------------------------------------------------------------
//	323	OUT p

void	Processor :: IO_OUT()
{
	union {
		PBYTE	c[2];
		PWORD	w;
		}		operand;

	operand.c[0] = operand.c[1] = Fetch();
	m_nBusAddress = operand.w;
	m_nBusWriteData = A;
	m_bIOWrite = true;
}

//	----------------------------------------------------------------------------
//	324	CNC w

void	Processor :: CNC()
{
	PWORD	w = (PWORD) (PC + 2);

	if (!CY)
	{
		SP -= 2;
		m_MemMap.Write2(SP, w);
		w = Fetch2();
	}
	PC = w;
}

//	----------------------------------------------------------------------------
//	325	PUSH D

void	Processor :: PUSH_D()
{
	SP -= 2;
	m_MemMap.Write2(SP, DE);
}

//	----------------------------------------------------------------------------
//	326	SUI n

void	Processor :: SUI()
{
	PBYTE	nTemp = A;
	PBYTE nOp = m_MemMap.Read(PC++);

	A = (PBYTE)(A - nOp);
	SetFlagsForSubtract(nTemp, nOp);
}

//	----------------------------------------------------------------------------
//	327	RST	2

void	Processor :: RST_2()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0010;
}

//	----------------------------------------------------------------------------
//	330	RC
void	Processor :: RC()
{
	if (CY)
	{
		PC = m_MemMap.Read2(SP);
		SP += 2;
	}
}

//	----------------------------------------------------------------------------
//	331	Varient

void	Processor :: VAR_331()
{
	m_nStatusOfVarientOp  = VarientOp(0331);
}

//	----------------------------------------------------------------------------
//	332	JC	w

void	Processor :: JC()
{
	PWORD	w = Fetch2();
	if (CY)
	{
		PC = w;
	}
}

//	----------------------------------------------------------------------------
//	333	IN	p

void	Processor :: IO_IN()
{
	union {
		PBYTE	c[2];
		PWORD	w;
		}		operand;
	operand.c[0] = operand.c[1] = Fetch();
	m_nBusAddress = operand.w;
	m_bIORead = true;
	m_bPendingRead = true;
	m_nInputTarget = REGNUM_A;
}

//	----------------------------------------------------------------------------
//	334	CC	w

void	Processor :: CC()
{
	PWORD	w = (PWORD) (PC + 2);

	if (CY)
	{
		SP -= 2;
		m_MemMap.Write2(SP, w);
		w = Fetch2();
	}
	PC = w;
}
//	----------------------------------------------------------------------------
//	335	Varient

void	Processor :: VAR_335()
{
	m_nStatusOfVarientOp  = VarientOp(0335);
}

//	----------------------------------------------------------------------------
//	336	SBI	n

void	Processor :: SBI()
{
	PBYTE	nBefore = A;
	PBYTE	nOp = m_MemMap.Read(PC++);
	PFLAG	bCY_OLD = CY;

	A = (PBYTE) (A - nOp - (bCY_OLD ? 1 : 0));
	SetFlagsForSBB(nBefore, nOp, bCY_OLD);
}

//	----------------------------------------------------------------------------
//	337	RST	3

void	Processor :: RST_3()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0018;
}

//	----------------------------------------------------------------------------
//	340	RPO

void	Processor :: RPO()
{
	m_nStatusOfVarientOp  = VarientOp(0340);
}

//	----------------------------------------------------------------------------
//	341	POP	H

void	Processor :: POP_H()
{
	HL = m_MemMap.Read2(SP);
	SP += 2;
}


//	----------------------------------------------------------------------------
//	342	JPO	w

void	Processor :: JPO()
{
	m_nStatusOfVarientOp  = VarientOp(0342);
}

//	----------------------------------------------------------------------------
//	343	XTHL

void	Processor :: XTHL()
{
	PWORD	w = m_MemMap.Read2(SP);
	m_MemMap.Write2(SP, HL);
	HL = w;
}

//	----------------------------------------------------------------------------
//	344	CPO	w

void	Processor :: CPO()
{
	m_nStatusOfVarientOp  = VarientOp(0344);
}

//	----------------------------------------------------------------------------
//	345	PUSH H

void	Processor :: PUSH_H()
{
	SP -= 2;
	m_MemMap.Write2(SP, HL);
}

//	----------------------------------------------------------------------------
//	346	ANI	n

void	Processor :: ANI()
{
	A = (PBYTE) (A & m_MemMap.Read(PC++));
	SetFlagsForAND();
}

//	--------------------------------------------------------------------------
//	347	RST	4

void	Processor :: RST_4()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0020;
}

//	----------------------------------------------------------------------------
//	350	RPE

void	Processor :: RPE()
{
	m_nStatusOfVarientOp  = VarientOp(0350);
}

//	----------------------------------------------------------------------------
//	351	PCHL

void	Processor :: PCHL()
{
	PC = HL;
}

//	----------------------------------------------------------------------------
//	352	JPE	w

void	Processor :: JPE()
{
	m_nStatusOfVarientOp  = VarientOp(0352);
}

//	----------------------------------------------------------------------------
//	353	XCHG

void	Processor :: XCHG()
{
	PWORD	w = DE;
	DE = HL;
	HL = w;
}

//	----------------------------------------------------------------------------
//	354	CPE	w

void	Processor :: CPE()
{
	m_nStatusOfVarientOp  = VarientOp(0354);
}

//	----------------------------------------------------------------------------
//	355	Varient

void	Processor :: VAR_355()
{
	m_nStatusOfVarientOp  = VarientOp(0355);
}

//	----------------------------------------------------------------------------
//	356	XRI	n

void	Processor :: XRI()
{
	A = (PBYTE) (A ^ m_MemMap.Read(PC++));
	SetFlagsForXOR();
}

//	----------------------------------------------------------------------------
//	357	RST	5

void	Processor :: RST_5()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0028;
}

//	----------------------------------------------------------------------------
//	360	RP

void	Processor :: RP()
{
	if (!S)
	{
		PC = m_MemMap.Read2(SP);
		SP += 2;
	}
}

//	----------------------------------------------------------------------------
//	361	POP	PSW

void	Processor :: POP_PSW()
{
	m_nStatusOfVarientOp = VarientOp(0361);
}

//	----------------------------------------------------------------------------
//	362	JP	w

void	Processor :: JP()
{
	PWORD	w = Fetch2();

	if (!S)
	{
		PC = w;
	}
}

//	----------------------------------------------------------------------------
//	363	DI

void	Processor :: DI()
{
	m_bPendingInterruptEnable = false;
	m_bInterruptsEnabled = false;
}

//	----------------------------------------------------------------------------
//	364	CP	w

void	Processor :: CP()
{
	PWORD	w = (PWORD) (PC + 2);

	if (!S)
	{
		SP -= 2;
		m_MemMap.Write2(SP, w);
		w = Fetch2();
	}
	PC = w;
}

//	----------------------------------------------------------------------------
//	365	PUSH PSW

void	Processor :: PUSH_PSW()
{
	m_nStatusOfVarientOp = VarientOp(0365);
}
	
//	----------------------------------------------------------------------------
//	366	ORI	n

void	Processor :: ORI()
{
	A = (PBYTE) (A | m_MemMap.Read(PC++));
	SetFlagsForOR();
}

//	----------------------------------------------------------------------------
//	367	RST	6

void	Processor :: RST_6()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0030;
}

//	----------------------------------------------------------------------------
//	370	RM

void	Processor :: RM()
{
	if (S)
	{
		PC = m_MemMap.Read2(SP);
		SP += 2;
	}
}

//	----------------------------------------------------------------------------
//	371	SPHL

void	Processor :: SPHL()
{
	SP = HL;
}

//	----------------------------------------------------------------------------
//	372	JM	w

void	Processor :: JM()
{
	PWORD	w = Fetch2();

	if (S)
	{
		PC = w;
	}
}

//	----------------------------------------------------------------------------
//	373	EI

void	Processor :: EI()
{
	m_bPendingInterruptEnable = true;
}

//	----------------------------------------------------------------------------
//	374	CM	w

void	Processor :: CM()
{
	PWORD	w = (PWORD) (PC + 2);

	if (S)
	{
		SP -= 2;
		m_MemMap.Write2(SP, w);
		w = Fetch2();
	}
}

//	----------------------------------------------------------------------------
//	375	Varient

void	Processor :: VAR_375()
{
	m_nStatusOfVarientOp = VarientOp(0375);
}

//	----------------------------------------------------------------------------
//	376	CPI	n

void	Processor :: CPI()
{
	SetFlagsForCompare(m_MemMap.Read(PC++));
}

//	----------------------------------------------------------------------------
//	377	RST	7

void	Processor :: RST_7()
{
	SP -= 2;
	m_MemMap.Write2(SP, PC);
	PC = 0x0038;
}


//	============================================================================
//	"helper functions" -- mostly to set flags after arithmetic & logical ops


void	Processor :: SetFlagsForINR(PBYTE cAfter)
{

//	Use cAfter (plus the fact that the operation was increment) to
//	set the processor flags.

//	CY unaffected.
	Z = (cAfter == 0);
	S = ((cAfter & 0x080) != 0);
	P = Parity(cAfter);
	AC = ((cAfter & 0x0F) == 0);
	
//	The 8085, needs to compute X5 and V; the Z-80 needs PV and AS

	V = (cAfter == 0x0080);
//	X5 = ???
	PV = V;	//	PV is overflow.
	AS = false;
}


void	Processor :: SetFlagsForDCR(PBYTE cAfter)
{

//	Use cAfter (plus the fact that the operation was decrement) to
//	set the processor flags.

//	CY unaffected
	Z = (cAfter == 0);
	S = ((cAfter & 0x080) != 0);
	P = Parity(cAfter);
	AC = ((cAfter & 0x0F) == 0x0F);
	
//	The 8085, needs to compute X5 and V; the Z-80 needs PV and AS

	V = (cAfter == 0x07f);
//	X5 = ???
	PV = V;		//	PV is overflow
	AS = true;
}


void	Processor :: SetFlagsForDAD(PWORD nOp1, PWORD nOp2, long nSum)
{
	CY = ((nSum & 0x0010000) != 0);

//	S unaffected
//	Z unaffected
//	P unaffected
//	V unaffected

//	The 8085, needs to compute X5 and V; the Z-80 needs PV and AS

//	X5 = sign_of_operand_1 && sign_of_operand_2 ||
//			sign_of_operand_1 && sign_of_result ||
//			sign_of_operand_2  && sign_of_result

	X5= (((nOp1 & 0x08000) && (nOp2 & 0x08000)) ||
		((nOp1 & 0x08000) && (nSum & 0x08000)) ||
		((nOp2 & 0x08000) && (nSum & 0x08000)));

//	PV unaffected
	AS = false;
}


void	Processor :: SetFlagsForAdd(PBYTE nBefore, PBYTE nOp)
{
	AC = (((nBefore & 0x08) + (nOp & 0x08)) == 0x10);
	CY = ((((nBefore & 0x0080) + (nOp & 0x0080)) & 0x0100) != 0);
	Z = (A == 0);
	S = ((A & 0x0080) != 0);
	P = Parity(A);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = (((nBefore ^ nOp ^ A) & 0x0080) != 0);
//	X5 = ??
	AS = false;
	PV = V; // PV is overflow.
}


void	Processor :: SetFlagsForADC(PBYTE nBefore, PBYTE nOp, PFLAG bCY_OLD)
{
	AC = (((nBefore & 0x08) + (nOp & 0x08)) == 0x10);
	CY = (((nBefore + nOp + (bCY_OLD ? 1 : 0)) & 0x0100) != 0);
	Z = (A == 0);
	S = ((A & 0x0080) != 0);
	P = Parity(A);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = (((nBefore ^ nOp ^ A) & 0x0080) != 0);
//	X5 = ??
	AS = false;
	PV = V; // PV is overflow.
}


void	Processor :: SetFlagsForSubtract(PBYTE nBefore, PBYTE nOp)
{
	AC = ((nBefore & 0x0f) > (nOp & 0x0F));
	CY = (nOp > nBefore);
	P = Parity(A);
	Z = (A == 0);
	S = ((A & 0x080) != 0);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = (((nBefore ^ nOp ^ A) & 0x0080) != 0);
//	X5 = ??
	AS = true;
	PV = V; // PV is overflow.
}


void	Processor :: SetFlagsForSBB(PBYTE nBefore, PBYTE nOp, PFLAG bCY_OLD)
{
	AC = ((nBefore & 0x0f) > (nOp & 0x0F));
	CY = (((nBefore - nOp - (bCY_OLD ? 1 : 0)) & 0x0100) != 0);
	P = Parity(A);
	Z = (A == 0);
	S = ((A & 0x080) != 0);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = (((nBefore ^ nOp ^ A) & 0x0080) != 0);
//	X5 = ??
	AS = true;
	PV = V; // PV is overflow.
}


void	Processor :: SetFlagsForAND()
{
	CY = false;
	AC = true;
	P = Parity(A);
	Z = (A == 0);
	S = ((A & 0x0080) != 0);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = false;
//	X5 = ???
	PV = P;		//	PV is parity
	AS = false;
}


void	Processor :: SetFlagsForOR()
{
	CY = false;
	AC = false;
	P = Parity(A);
	Z = (A == 0);
	S = ((A & 0x0080) != 0);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = false;
//	X5 = ???
	PV = P;		//	PV is parity
	AS = false;
}


void	Processor :: SetFlagsForXOR()
{
	CY = false;
	AC = false;
	P = Parity(A);
	Z = (A == 0);
	S = ((A & 0x0080) != 0);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = false;
//	X5 = ???
	PV = P;		//	PV is parity
	AS = false;
}


void	Processor :: SetFlagsForCompare(PBYTE nOp)
{
	PBYTE	nTemp = (PBYTE)(A - nOp);

	AC = ((A & 0x0f) > (nOp & 0x0F));
	CY = nOp > A;
	P = Parity(nTemp);
	Z = (nTemp == 0);
	S = ((nTemp & 0x080) != 0);

//	The 8085, needs to compute X5 and V; the Z-80 needs Z5, Z3, PV and AS

	V = (((A ^ nOp ^ nTemp) & 0x0080) != 0);
//	X5 = ??
	AS = true;
	PV = V; // PV is overflow.
}


inline PFLAG	Processor :: Parity(PBYTE c)
{

//	P is set if c has an even number of 1 bits.

	PBYTE c1 = (PBYTE)(c ^ (c >> 1));
	PBYTE c2 = (PBYTE)(c1 ^ (c1 >> 2));
	return !((c2 ^ (c2 >> 4)) & 0x01);
}


//	============================================================================
//	Some old-technology opcode routines (will eventually go away):

void	Processor :: InputToRegister(int nRegNum, PBYTE c)
{

//	TODO:	If the target is not REGNUM_A, the operation is a Z-80 I/O Input.
//	For those operations, we should also affect the flags.


	switch(nRegNum)
	{
	case REGNUM_B:
		B = c;
		break;
	case REGNUM_C:
		C = c;
		break;
	case REGNUM_D:
		D = c;
		break;
	case REGNUM_E:
		E = c;
		break;
	case REGNUM_H:
		H = c;
		break;
	case REGNUM_L:
		L = c;
		break;
	case REGNUM_A:
		A = c;
		break;
	case INPUT_TO_A_WITH_FLAGS:
		A = c;
		break;
	case INPUT_TO_M_AND_DECREMENT_ONCE:
		m_MemMap.Write(HL--, c);
		DCR_B();
		break;
	case INPUT_TO_M_AND_INCREMENT_ONCE:
		m_MemMap.Write(HL++, c);
		DCR_B();
		break;

	default:

//	TODO:	There is one undocumented I/O read of the Z-80 which does not save
//	the data but which does set the flags.

		break;
	}
}
