/************************************************************************
 *																		*
 *	H8 EMULATOR Project --  PROCESSOR.H									*
 *																		*
 *	A simulation of the 8080, 8085 and Z80 microprocessors				*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *										01863-1202						*
 *																		*
 ************************************************************************/


#ifndef	_PROCESSOR_H_
#define	_PROCESSOR_H_


#ifndef _H8TYPES_H_
#include "h8types.h"
#endif

#ifndef _MEMMAP_H_
#include "memmap.h"
#endif


//	============================================================================


//	Synonyms for register names:

#define	B	rpBC.reg[1]
#define	C	rpBC.reg[0]
#define	BC	rpBC.regpair
#define	D	rpDE.reg[1]
#define	E	rpDE.reg[0]
#define	DE	rpDE.regpair
#define	H	rpHL.reg[1]
#define	L	rpHL.reg[0]
#define	HL	rpHL.regpair
#define	B1	rpBC1.reg[1]
#define	C1	rpBC1.reg[0]
#define	BC1	rpBC1.regpair
#define	D1	rpDE1.reg[1]
#define	E1	rpDE1.reg[0]
#define	DE1	rpDE1.regpair
#define	H1	rpHL1.reg[1]
#define	L1	rpHL1.reg[0]
#define	HL1	rpHL1.regpair
#define	I	rpIR.reg[0]
#define	R	rpIR.reg[1]

//	Execution result codes:

#define HALTED			1
#define ILLEGAL_OP		2
#define UNIMPLEMENTED	3
#define TERMINATE		4
#define RESET			5
#define	BREAK_POINT	   99
#define ABORT		  999

//	============================================================================


//	The base class:

class Processor
{
protected:
	union REGPAIR {
		PBYTE		reg[2];
		PWORD		regpair;
	};

public:
	Processor();
	~Processor();

public:
	virtual void	Reset();
	int				Execute();
	int				Step();
	__inline PBYTE	Fetch();
	__inline PWORD	Fetch2();
	__inline PFLAG	Parity(PBYTE c);
	CString			Disassemble(PWORD nPC);

protected:
	void			SubtractRegisterPairs(PWORD * pTarget, PWORD * pOperand);
	virtual int		VarientOp(PBYTE nOP);
	virtual CString	DisassembleVarientOp(PBYTE nOp,
											short int nW,
											short int nX);

private:
	void			SetFlagsForINR(PBYTE cAfter);
	void			SetFlagsForDCR(PBYTE cAfter);
	void			SetFlagsForAdd(PBYTE nBefore, PBYTE nOp);
	void			SetFlagsForDAD(PWORD nOp1, PWORD nOp2, long nSum);
	void			SetFlagsForADC(PBYTE nBefore, PBYTE nOp, PFLAG bCY_OLD);
	void			SetFlagsForSubtract(PBYTE nBefore, PBYTE nOp);
	void			SetFlagsForSBB(PBYTE nBefore, PBYTE nOp, PFLAG bCY_OLD);
	void			SetFlagsForAND();
	void			SetFlagsForXOR();
	void			SetFlagsForOR();
	void			SetFlagsForCompare(PBYTE nOp);

//	These methods implement the "baseline" opcode set:
	
	void			NOP();
	void			LXI_B();
	void			STAX_B();
	void			INX_B();
	void			INR_B();
	void			DCR_B();
	void			MVI_B();
	void			RLC();
	void			VAR_010();
	void			DAD_B();
	void			LDAX_B();
	void			DCX_B();
	void			INR_C();
	void			DCR_C();
	void			MVI_C();
	void			RRC();
	void			VAR_020();
	void			LXI_D();
	void			STAX_D();
	void			INX_D();
	void			INR_D();
	void			DCR_D();
	void			MVI_D();
	void			RAL();
	void			VAR_030();
	void			DAD_D();
	void			LDAX_D();
	void			DCX_D();
	void			INR_E();
	void			DCR_E();
	void			MVI_E();
	void			RAR();
	void			VAR_040();
	void			LXI_H();
	void			SHLD();
	void			INX_H();
	void			INR_H();
	void			DCR_H();
	void			MVI_H();
	void			DAA();
	void			VAR_050();
	void			DAD_H();
	void			LHLD();
	void			DCX_H();
	void			INR_L();
	void			DCR_L();
	void			MVI_L();
	void			CMA();
	void			VAR_060();
	void			LXI_SP();
	void			STA();
	void			INX_SP();
	void			INR_M();
	void			DCR_M();
	void			MVI_M();
	void			STC();
	void			VAR_070();
	void			DAD_SP();
	void			LDA();
	void			DCX_SP();
	void			INR_A();
	void			DCR_A();
	void			MVI_A();
	void			CMC();
	void			MOV_BB();
	void			MOV_BC();
	void			MOV_BD();
	void			MOV_BE();
	void			MOV_BH();
	void			MOV_BL();
	void			MOV_BM();
	void			MOV_BA();
	void			MOV_CB();
	void			MOV_CC();
	void			MOV_CD();
	void			MOV_CE();
	void			MOV_CH();
	void			MOV_CL();
	void			MOV_CM();
	void			MOV_CA();
	void			MOV_DB();
	void			MOV_DC();
	void			MOV_DD();
	void			MOV_DE();
	void			MOV_DH();
	void			MOV_DL();
	void			MOV_DM();
	void			MOV_DA();
	void			MOV_EB();
	void			MOV_EC();
	void			MOV_ED();
	void			MOV_EE();
	void			MOV_EH();
	void			MOV_EL();
	void			MOV_EM();
	void			MOV_EA();
	void			MOV_HB();
	void			MOV_HC();
	void			MOV_HD();
	void			MOV_HE();
	void			MOV_HH();
	void			MOV_HL();
	void			MOV_HM();
	void			MOV_HA();
	void			MOV_LB();
	void			MOV_LC();
	void			MOV_LD();
	void			MOV_LE();
	void			MOV_LH();
	void			MOV_LL();
	void			MOV_LM();
	void			MOV_LA();
	void			MOV_MB();
	void			MOV_MC();
	void			MOV_MD();
	void			MOV_ME();
	void			MOV_MH();
	void			MOV_ML();
	void			HLT();
	void			MOV_MA();
	void			MOV_AB();
	void			MOV_AC();
	void			MOV_AD();
	void			MOV_AE();
	void			MOV_AH();
	void			MOV_AL();
	void			MOV_AM();
	void			MOV_AA();
	void			ADD_B();
	void			ADD_C();
	void			ADD_D();
	void			ADD_E();
	void			ADD_H();
	void			ADD_L();
	void			ADD_M();
	void			ADD_A();
	void			ADC_B();
	void			ADC_C();
	void			ADC_D();
	void			ADC_E();
	void			ADC_H();
	void			ADC_L();
	void			ADC_M();
	void			ADC_A();
	void			SUB_B();
	void			SUB_C();
	void			SUB_D();
	void			SUB_E();
	void			SUB_H();
	void			SUB_L();
	void			SUB_M();
	void			SUB_A();
	void			SBB_B();
	void			SBB_C();
	void			SBB_D();
	void			SBB_E();
	void			SBB_H();
	void			SBB_L();
	void			SBB_M();
	void			SBB_A();
	void			ANA_B();
	void			ANA_C();
	void			ANA_D();
	void			ANA_E();
	void			ANA_H();
	void			ANA_L();
	void			ANA_M();
	void			ANA_A();
	void			XRA_B();
	void			XRA_C();
	void			XRA_D();
	void			XRA_E();
	void			XRA_H();
	void			XRA_L();
	void			XRA_M();
	void			XRA_A();
	void			ORA_B();
	void			ORA_C();
	void			ORA_D();
	void			ORA_E();
	void			ORA_H();
	void			ORA_L();
	void			ORA_M();
	void			ORA_A();
	void			CMP_B();
	void			CMP_C();
	void			CMP_D();
	void			CMP_E();
	void			CMP_H();
	void			CMP_L();
	void			CMP_M();
	void			CMP_A();
	void			RNZ();
	void			POP_B();
	void			JNZ();
	void			JMP();
	void			CNZ();
	void			PUSH_B();
	void			ADI();
	void			RST_0();
	void			RZ();
	void			RET();
	void			JZ();
	void			VAR_313();
	void			CZ();
	void			CALL();
	void			ACI();
	void			RST_1();
	void			RNC();
	void			POP_D();
	void			JNC();
	void			IO_OUT();
	void			CNC();
	void			PUSH_D();
	void			SUI();
	void			RST_2();
	void			RC();
	void			VAR_331();
	void			JC();
	void			IO_IN();
	void			CC();
	void			VAR_335();
	void			SBI();
	void			RST_3();
	void			RPO();
	void			POP_H();
	void			JPO();
	void			XTHL();
	void			CPO();
	void			ANI();
	void			PUSH_H();
	void			RST_4();
	void			RPE();
	void			PCHL();
	void			JPE();
	void			XCHG();
	void			CPE();
	void			VAR_355();
	void			XRI();
	void			RST_5();
	void			RP();
	void			POP_PSW();
	void			JP();
	void			DI();
	void			CP();
	void			PUSH_PSW();
	void			ORI();
	void			RST_6();
	void			RM();
	void			SPHL();
	void			JM();
	void			EI();
	void			CM();
	void			VAR_375();
	void			CPI();
	void			RST_7();

//	these functions will go away...

	void		InputToRegister(int nRegNum, PBYTE c);

public:
	MemoryMap		m_MemMap;

//	Registers and flags for all three processors:

	PFLAG			CY, V, P, Z3, AC, X5, Z5, Z, S, AS, PV;
	PFLAG			CY1, AS1, PV1, Z31, AC1, Z51, Z1, S1;
	PBYTE			A;
	PBYTE			A1;
	union REGPAIR	rpBC, rpDE, rpHL;
	union REGPAIR	rpBC1, rpDE1, rpHL1;
	PWORD			PC;
	PWORD			IX;
	PWORD			IY;
	PWORD			SP;
	union REGPAIR	rpIR;
	PBYTE *			M;

//	address bus:

	short int		m_nBusAddress;

//	"input-side" of data bus:

	char			m_nBusReadData;
	int				m_nInputTarget;

//	"output-side" of data bus:

	char			m_nBusWriteData;

//	Control signals that govern IO read and IO write cycles:

	bool			m_bIORead;
	bool			m_bIOWrite;

//	Control signals that govern interrupt request and interrupt acknowledge

	bool			m_bIntReq;
	bool			m_bIntAck;

//	Status signals

	bool			m_bInterruptsEnabled;
	bool			m_bRun;
	bool			m_bM1;

//	These (I8085) signals are not "connected".

	bool			m_bSIN;
	bool			m_bSOUT;
	bool			m_bNMI;
	bool			m_bRST5_5;
	bool			m_bRST6_5;
	bool			m_bRST7_5;

//	Flags needed by the simulation -- no corresponding hardware.

	bool			m_bInterruptFetch;
	bool			m_bPendingRead;

protected:

//	Internal state flag for the interrupt enable (which is delayed a clock)

	bool			m_bPendingInterruptEnable;

private:
	int				m_nStatusOfVarientOp;
};

//	============================================================================


//	The implementation for the 8080 processor:

class I8080: public Processor
{
public:
	I8080();
	~I8080();
	void			Reset();
	int				VarientOp(PBYTE nOP);
	CString			DisassembleVarientOp(PBYTE nOp, short int nW, short int nX);
protected:
private:

public:
};

//	============================================================================


//	The implementation for the 8085 processor:

class I8085: public Processor
{
public:
	I8085();
	~I8085();
	void			Reset();
	int				VarientOp(PBYTE nOP);
	CString			DisassembleVarientOp(PBYTE nOp,
											short int nW,
											short int nX);
protected:
private:
};

//	============================================================================


//	The implementation for the Z80 processor:

class Z80: public Processor
{
public:
	Z80();
	~Z80();
	void			Reset();
	int				VarientOp(PBYTE nOP);
	CString			DisassembleVarientOp(PBYTE nOp, short int nW, short int nX);
protected:
private:

//	Attributes:

public:
protected:
private:
};

#endif	//	#ifndef	_PROCESSOR_H_
