/************************************************************************
 *																		*
 *	H8 EMULATOR Project --	VIRTUAL_H8VIEW.H							*
 *																		*
 *	User Interface for the Heathkit H-8 simulator project				*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/

#if !defined(AFX_VIRTUAL_H8VIEW_H__C9847060_7201_11D5_9C70_005004AF5E32__INCLUDED_)
#define AFX_VIRTUAL_H8VIEW_H__C9847060_7201_11D5_9C70_005004AF5E32__INCLUDED_


#include "display.h"
#include "bus.h"
#include "processor.h"
#include "panelbutton.h"


#ifndef _SCREENWINDOW_H_
#include "screenwindow.h"
#endif


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class CVirtual_h8View : public CFormView
{

//	Constructor / Destructor:

protected: // create from serialization only
	CVirtual_h8View();
	DECLARE_DYNCREATE(CVirtual_h8View)
	~CVirtual_h8View();


//	Class Wizard stuff:

//	Generated Methods:
public:
	CVirtual_h8Doc* GetDocument();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

// Overrides
public:
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CVirtual_h8View)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void OnInitialUpdate(); // called first time after construct
	//}}AFX_VIRTUAL

// Generated message map functions
protected:
	//{{AFX_MSG(CVirtual_h8View)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnKbdHelpCk();
	afx_msg void OnLocal();
	afx_msg void OnViewP3();
	afx_msg void OnUpdateViewP3(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewP2(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewP1(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewP0(CCmdUI* pCmdUI);
	afx_msg void OnConfigH19();
	afx_msg void OnViewP2();
	afx_msg void OnViewP1();
	afx_msg void OnViewP0();
	afx_msg void OnH85();
	afx_msg void OnUpdateH85(CCmdUI* pCmdUI);
	afx_msg void OnConfigH8();
	//}}AFX_MSG
	afx_msg LRESULT OnIncoming(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnCharToH19(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnBreakToH19(WPARAM wParam, LPARAM lParam);
	DECLARE_MESSAGE_MAP()

//	Dialog Data
public:
	//{{AFX_DATA(CVirtual_h8View)
	enum { IDD = IDD_VIRTUAL_H8_FORM };
	CStatic	m_H19Frame;
	CStatic	m_H8Frame;
	CStatic	m_CPThreadStatusWindow;
	CStatic	m_IOThreadStatusWindow;
	CStatic	m_KeyhelpGraphic;
	CButton	m_KbdHelpBtn;
	CButton	m_Local;
	CStatic	m_Nameplate;
	//}}AFX_DATA


//	Methods:

public:
protected:
private:
	void		ConfigLogicAnalyzer();
	void		MessageDispatch(CString strMsg, int nConnection);
	void		ClearAllLocks();
	void		FunctionKey(int nKeyCode);
	void		KeypadKey(int nKeyCode);
	void		Spacebar();
	void		ScrollLock();
	void		Break();
	void		CtrlChar(int nKeyCode);
	void		EscapeChar();
	void		Character(int nKeyCode);
	int			TestLockAndShiftState();
	void		CalculateLayout();


//	Attributes:

public:
	int				m_nPhotorealismMode;
	CString			m_strIniFile;

protected:
private:
	Display			m_wDisplay;
	PanelButton		m_Keypad1Btn;
	PanelButton		m_Keypad2Btn;
	PanelButton		m_Keypad3Btn;
	PanelButton		m_Keypad4Btn;
	PanelButton		m_Keypad5Btn;
	PanelButton		m_Keypad6Btn;
	PanelButton		m_Keypad7Btn;
	PanelButton		m_Keypad8Btn;
	PanelButton		m_Keypad9Btn;
	PanelButton		m_Keypad10Btn;
	PanelButton		m_Keypad11Btn;
	PanelButton		m_Keypad12Btn;
	PanelButton		m_Keypad13Btn;
	PanelButton		m_Keypad14Btn;
	PanelButton		m_Keypad15Btn;
	PanelButton		m_Keypad16Btn;
	PanelButton		m_Keypad17Btn;
	PanelButton		m_Keypad18Btn;
	CStatic *		m_pMagnetButton;
	CBitmap			m_MagnetBmp;
	CBitmap			m_NoMagnetBmp;
	int				m_nTimerID;

	ScreenWindow	m_Screen;
	BOOL			m_bLocal;
	BOOL			m_bCapsLock;
	BOOL			m_bScrollLock;
	BOOL			m_bNumLock;
	double			m_dMagnification;
	CStringList		m_cslConsoleOutputBuffer;

};


void		OnResetButton(BOOL bState);
void		OnSevenButton(BOOL bState);
void		OnEightButton(BOOL bState);
void		OnNineButton(BOOL bState);
void		OnAButton(BOOL bState);
void		OnFourButton(BOOL bState);
void		OnFiveButton(BOOL bState);
void		OnSixButton(BOOL bState);
void		OnBButton(BOOL bState);
void		OnOneButton(BOOL bState);
void		OnTwoButton(BOOL bState);
void		OnThreeButton(BOOL bState);
void		OnCButton(BOOL bState);
void		OnZeroButton(BOOL bState);
void		OnFButton(BOOL bState);
void		OnEButton(BOOL bState);
void		OnDButton(BOOL bState);
void		OnRTMButton(BOOL bState);

#ifndef _DEBUG  // debug version in virtual_h8View.cpp
inline CVirtual_h8Doc* CVirtual_h8View::GetDocument()
   { return (CVirtual_h8Doc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_VIRTUAL_H8VIEW_H__C9847060_7201_11D5_9C70_005004AF5E32__INCLUDED_)
